!+
 MODULE getInputs !==========================================================
! ---------------------------------------------------------------------------
! PURPOSE - Retrieve input data from inputs.txt file, perform initial
!           checks and load global arrays
!
!
! AUTHOR  - Marco Montalto, Centro de Astrofísica da Universidade do Porto,
!                           (CAUP)
!
!
!     REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
!    ??    1.0   MM    Original coding                     
!
 IMPLICIT NONE
 PRIVATE

 INTEGER, PARAMETER, PUBLIC    :: MAXOBJECTS=100
 INTEGER, PARAMETER, PUBLIC    :: DP=KIND(1.0D0)
 REAL(DP), PARAMETER, PUBLIC   :: PI = 4.*atan(1.)    
 REAL(DP), PARAMETER, PUBLIC   :: HPI = 2.*atan(1.)    
 REAL(DP), PARAMETER, PUBLIC   :: TWPI = 8.*atan(1.)    
 REAL(DP), PARAMETER, PUBLIC   :: epsilon=1.E-02

 TYPE, PUBLIC                  :: simparam_
  INTEGER                      :: nsteps = 0, nplanets = 0, nspots = 0, nobj=0
  REAL(DP)                     :: timestep = 0.0D0
 END TYPE simparam_
 TYPE, PUBLIC                  :: starparam_
  REAL(DP)                     :: prot = 0.0D0, incl = 0.0D0, posang = 0.0D0, rho = 0.0D0
  REAL(DP)                     :: c1 = 0.0D0, c2 = 0.0D0
 END TYPE starparam_
 TYPE, PUBLIC                  :: spots_
  INTEGER, POINTER             :: id(:) => null()
  REAL(DP), POINTER            :: lat(:) => null(), long(:) => null(), adim(:) => null()
  REAL(DP), POINTER            :: ffact(:) => null()
 END TYPE spots_
 TYPE, PUBLIC                  :: planets_
  INTEGER, POINTER             :: id(:) => null()
  REAL(DP), POINTER            :: rp(:) => null(), porb(:) => null(), pincl(:) => null()
  REAL(DP), POINTER            :: ecc(:) => null(), w(:) => null(), M(:) => null(), omega(:)=>null()
 END TYPE planets_
 TYPE, PUBLIC                  :: res_
  INTEGER, POINTER             :: status(:) => null()
  REAL(DP), POINTER            :: time(:) => null(), flux(:) => null()
 END TYPE res_
 PUBLIC                        :: unload
 PUBLIC                        :: load
 PUBLIC                        :: errmsg
 PUBLIC                        :: rad
 PUBLIC                        :: deg
 PUBLIC                        :: check_inputs

CONTAINS

SUBROUTINE simload(simp)
  IMPLICIT NONE
  INTEGER            :: lun = 10, err, nstep, nplanets, nspots, nsteps
  REAL(DP)           :: timestep
  TYPE(simparam_)    :: simp
  NAMELIST /SIMPARAM/ timestep, nsteps, nplanets, nspots
  OPEN(lun,FILE='inputs.txt',STATUS='old',IOSTAT=err)
  IF (err/=0) then
    CALL errmsg(1)
  ENDIF
       READ(lun,NML=SIMPARAM,IOSTAT=err)
       IF (err/=0) then
         CALL errmsg(2)
       ENDIF
       simp%timestep = timestep
       simp%nsteps   = nsteps
       simp%nspots   = nspots
       simp%nplanets = nplanets
       simp%nobj = nplanets+nspots
  CLOSE(lun)
END SUBROUTINE simload

SUBROUTINE loadstar(starp)
  IMPLICIT NONE
  INTEGER            :: lun = 10, err
  REAL(DP)           :: prot=0.0D0, incl=0.0D0, posang=0.0D0, rho=0.0D0, c1=0.0D0, c2=0.0D0
  TYPE(starparam_)   :: starp
  NAMELIST /STARPARAM/ prot, incl, posang, rho, c1, c2
  OPEN(lun,FILE='inputs.txt',STATUS='old',IOSTAT=err)
  IF (err/=0) then
    CALL errmsg(1)
  ENDIF
       READ(lun,NML=STARPARAM,IOSTAT=err)
       IF (err/=0) then
           CALL errmsg(4)
       ENDIF
       starp%prot   = prot
       starp%incl   = incl
       starp%posang = posang
       starp%rho    = rho
       starp%c1     = c1
       starp%c2     = c2
  CLOSE(lun)
END SUBROUTINE loadstar

SUBROUTINE loadspots(sp,nspots)
  IMPLICIT NONE
  INTEGER            :: i, lun = 10, err, nspots
  INTEGER            :: id(MAXOBJECTS)=0
  REAL(DP)           :: adim(MAXOBJECTS)=0.0D0, lat(MAXOBJECTS)=0.0D0, long(MAXOBJECTS)=0.0D0 
  REAL(DP)           :: ffact(MAXOBJECTS)=0.0D0
  TYPE(spots_)       :: sp
  NAMELIST /SPOTS/ lat, long, adim, ffact
  OPEN(lun,FILE='inputs.txt',STATUS='old',IOSTAT=err)
       IF (err/=0) then
           CALL errmsg(1)
       ENDIF
       READ(lun,NML=SPOTS,IOSTAT=err)
       IF (err/=0) then
           CALL errmsg(5)
       ENDIF
       do i=1, nspots
          sp%id(i)     = i
          sp%adim(i)   = adim(i)
          sp%lat(i)    = lat(i)
          sp%long(i)   = long(i)
          sp%ffact(i)  = ffact(i)
       enddo
  CLOSE(lun)
END SUBROUTINE loadspots

SUBROUTINE loadplanets(pl,nplanets)
  IMPLICIT NONE
  INTEGER            :: i, lun = 10, err, nplanets, id(MAXOBJECTS)=0
  REAL(DP)           :: rp(MAXOBJECTS)=0.0D0, porb(MAXOBJECTS)=0.0D0, pincl(MAXOBJECTS)=0.0D0
  REAL(DP)           :: ecc(MAXOBJECTS)=0.0D0, w(MAXOBJECTS)=0.0D0, M(MAXOBJECTS)=0.0D0
  REAL(DP)           :: omega(MAXOBJECTS)=0.0D0
  TYPE(planets_)     :: pl
  NAMELIST /PLANETS/ rp, porb, pincl, ecc, w, omega, M
  OPEN(lun,FILE='inputs.txt',STATUS='old',IOSTAT=err)
       IF (err/=0) then
           CALL errmsg(1)
       ENDIF
       READ(lun,NML=PLANETS,IOSTAT=err)
       IF (err/=0) then
           CALL errmsg(6)
       ENDIF
       do i=1, nplanets
          pl%id(i)    = i
          pl%rp(i)    = rp(i)
          pl%porb(i)  = porb(i)
          pl%pincl(i) = pincl(i)
          pl%ecc(i)   = ecc(i)
          pl%w(i)     = w(i)
          pl%omega(i) = omega(i)
          pl%M(i)     = M(i)
       enddo
  CLOSE(lun)
END SUBROUTINE loadplanets

SUBROUTINE check_inputs(simp,starp,planets,spots)
IMPLICIT NONE

INTEGER             :: nobj, i, j, nspots, nplanets
TYPE(simparam_)     :: simp
TYPE(spots_)        :: spots, spotscp
TYPE(planets_)      :: planets, planetscp
TYPE(starparam_)    :: starp
REAL(DP)            :: cos_rho
REAL(DP)            :: adim, lat, long, ffact

IF (simp%nsteps<1) THEN
   call errmsg(7)  
ENDIF
IF (simp%timestep<0.0D0) THEN
   call errmsg(8)  
ENDIF
IF (starp%prot<0.0D0) THEN
   call errmsg(9)
ENDIF
IF ((starp%incl<0.0D0).OR.(starp%incl>180.0D0)) THEN
   call errmsg(10)
ENDIF
IF ((starp%posang<0.0D0).OR.(starp%posang>360.0D0)) THEN
   call errmsg(11)
ENDIF
IF (starp%rho<0.) THEN
   call errmsg(12)
ENDIF
IF ((starp%c1<0.0D0).OR.(starp%c1>1.0D0)) THEN
   call errmsg(13)
endif
IF ((starp%c2<0.0D0).OR.(starp%c2>1.0D0)) THEN
   call errmsg(14)
ENDIF

IF (simp%nplanets>0) THEN
DO i=1, simp%nplanets
    IF (planets%porb(i)<0.0D0) THEN
      print*,'Planet N.',i              
      call errmsg(16)
    ENDIF
    IF ((planets%pincl(i)<-180.0D0).OR.(planets%pincl(i)>180.0D0)) THEN
      print*,'Planet N.',i
      call errmsg(17)
    ENDIF
    IF ((planets%ecc(i).lt.0.0D0).OR.(planets%ecc(i).ge.1.0D0)) THEN
      print*,'Planet N.',i
      call errmsg(18)
    ENDIF
    IF ((planets%w(i)<0.0D0).OR.(planets%w(i)>360.0D0)) THEN
      print*,'Planet N.',i
      call errmsg(19)
    ENDIF
    IF ((planets%omega(i)<0.0D0).OR.(planets%omega(i)>360.0D0)) THEN
      print*,'Planet N.',i
      call errmsg(20)
    ENDIF
    IF ((planets%M(i)<0.0D0).OR.(planets%M(i)>360.0D0)) THEN
      print*,'Planet N.',i
      call errmsg(21)
    ENDIF
    IF (planets%rp(i)<=0.0D0) THEN
      print*,'Planet N.',i
      call errmsg(22)
    ENDIF
ENDDO

! Coincident planets
planetscp=planets
nplanets=simp%nplanets
DO i=1, nplanets-1
   DO j=i+1, nplanets
      IF ((planets%porb(i)==planets%porb(j)).AND.(planets%pincl(i)==planets%pincl(j)).AND. &
          (planets%ecc(i)==planets%ecc(j)).AND.(planets%w(i)==planets%w(j)).AND.(planets%M(i)==planets%M(j)).AND. &
          (planets%rp(i)==planets%rp(j))) THEN
          call dropplanet(planets%id(j),planetscp,nplanets)
      ENDIF
   ENDDO
ENDDO
planets=planetscp
simp%nplanets=nplanets
ENDIF 

IF (simp%nspots>0) THEN
 DO i=1, simp%nspots       
    IF ((spots%lat(i)<-90.0D0).OR.(spots%lat(i)>90.0D0)) THEN
      print*,'Spot N.',i
      CALL errmsg(23)
    ENDIF
    IF ((spots%long(i)<0.0D0).OR.(spots%long(i)>360.0D0)) THEN
      print*,'Spot N.',i
      CALL errmsg(24)
    ENDIF
    IF ((spots%adim(i)<=0.0D0).OR.(spots%adim(i)>180.0D0)) THEN
      print*,'Spot N.',i
      CALL errmsg(25)
    ENDIF
    IF ((spots%ffact(i)==0.0D0).OR.(spots%ffact(i)>1.0D0)) THEN
      print*,'Spot N.',i
      call errmsg(15)
    ENDIF
 ENDDO

! Spots inside/coincident with other spots with the same contrast ratio or anti-circumpolar spots.
 nspots=simp%nspots
 spotscp=spots
 IF (simp%nspots>1) THEN
   DO i=1, simp%nspots-1
     DO j=i+1, simp%nspots
        IF (i/=j) THEN
          cos_rho=cos(HPI-rad(spots%lat(i)))*cos(HPI-rad(spots%lat(j)))+sin(HPI-rad(spots%lat(i)))* & 
                  sin(HPI-rad(spots%lat(j)))*cos(rad(spots%long(i)-spots%long(j)))
          IF (cos_rho>=cos(rad(ABS(spots%adim(i)-spots%adim(j)))).AND.(spots%ffact(i)==spots%ffact(j))) THEN
            IF (spots%adim(i).GE.spots%adim(j)) THEN
              CALL dropspot(spots%id(j),spotscp,nspots)
            ELSE
              CALL dropspot(spots%id(i),spotscp,nspots)
            ENDIF                          
          ENDIF
          IF ((cos_rho==1.0D0).AND.(spots%adim(i)==spots%adim(j))) THEN
              CALL dropspot(spots%id(j),spotscp,nspots)          
          ENDIF
        ENDIF
     ENDDO
   ENDDO
 ENDIF

 simp%nspots=nspots
 spots=spotscp
 IF (simp%nspots>0) THEN
  DO i=1, simp%nspots
     IF ((starp%incl<90.0D0.AND.((spots%lat(i)+spots%adim(i))<=-starp%incl)).OR. &
         (starp%incl>90.0D0.AND.((spots%lat(i)-spots%adim(i))>=(starp%incl-90.0D0)))) THEN
          CALL dropspot(spots%id(i),spotscp,nspots)
     ENDIF
  ENDDO
 ENDIF
 simp%nspots=nspots
 spots=spotscp

! Sort spots for increasing constrast ratio
 IF (simp%nspots>1) THEN
  DO i=1, simp%nspots
     DO j=i+1, simp%nspots
        IF (spots%ffact(j)<spots%ffact(i)) THEN

            adim=spots%adim(j)
            lat=spots%lat(j)
            long=spots%long(j)
            ffact=spots%ffact(j)

            spots%adim(j)=spots%adim(i)
            spots%lat(j)=spots%lat(i)
            spots%long(j)=spots%long(i)
            spots%ffact(j)=spots%ffact(i)

            spots%adim(i)=adim
            spots%lat(i)=lat
            spots%long(i)=long
            spots%ffact(i)=ffact
           
        ENDIF
     ENDDO
  ENDDO
 ENDIF

ENDIF
END SUBROUTINE check_inputs

FUNCTION rad(angledeg) RESULT(anglerad)
 IMPLICIT NONE

 REAL(DP)                :: angledeg, anglerad
 anglerad=angledeg*PI/180.0D0
 RETURN
END FUNCTION rad

FUNCTION deg(anglerad) RESULT(angledeg)
 IMPLICIT NONE

 INTEGER, PARAMETER   :: DP=KIND(1.0D0)
 REAL(DP)             :: angledeg, anglerad
 angledeg=anglerad*180.0D0/PI
 RETURN
END FUNCTION deg

SUBROUTINE dropspot(id,spots,nspots)
 IMPLICIT NONE

 INTEGER             :: id, i, j, nspots, idx
 TYPE(spots_)        :: spots

 IF (nspots.gt.0) THEN

 DO i=1, nspots
  IF (spots%id(i)==id) THEN
      idx=i
      EXIT
  ENDIF
 ENDDO

   IF (idx.eq.nspots) THEN
       spots%id(idx)=0
       spots%lat(idx)=0.0D0
       spots%long(idx)=0.0D0
       spots%adim(idx)=0.0D0
       spots%ffact(idx)=0.0D0
       nspots=nspots-1
   ELSE IF ((idx.lt.nspots).AND.(idx.gt.0)) THEN
     DO j=i+1, nspots
       spots%id(idx)=spots%id(j)
       spots%lat(idx)=spots%lat(j)
       spots%long(idx)=spots%long(j)
       spots%adim(idx)=spots%adim(j)
       spots%ffact(idx)=spots%ffact(j)
     ENDDO
       spots%id(nspots)=0
       spots%lat(nspots)=0.0D0
       spots%long(nspots)=0.0D0
       spots%adim(nspots)=0.0D0
       spots%ffact(nspots)=0.0D0
       nspots=nspots-1
   ENDIF
 ENDIF
END SUBROUTINE dropspot

SUBROUTINE dropplanet(id,planets,nplanets)
 IMPLICIT NONE

 INTEGER             :: id, idx, i, j, nplanets
 TYPE(planets_)      :: planets

 IF (nplanets.gt.0) THEN

 DO i=1, nplanets
  IF (planets%id(i)==id) THEN
      idx=i
      EXIT
  ENDIF
 ENDDO

   IF (idx.eq.nplanets) THEN
       planets%id(idx)=0
       planets%porb(idx)=0.0D0
       planets%pincl(idx)=0.0D0
       planets%ecc(idx)=0.0D0
       planets%w(idx)=0.0D0
       planets%M(idx)=0.0D0
       planets%rp(idx)=0.0D0
       nplanets=nplanets-1
   ELSE IF ((idx.lt.nplanets).AND.(idx.gt.0)) THEN
     DO j=i+1, nplanets
       planets%id(idx)=planets%id(j)
       planets%porb(idx)=planets%porb(j)
       planets%pincl(idx)=planets%pincl(j)
       planets%ecc(idx)=planets%ecc(j)
       planets%w(idx)=planets%w(j)
       planets%M(idx)=planets%M(j)
       planets%rp(idx)=planets%rp(j)
     ENDDO
       planets%id(nplanets)=0
       planets%porb(nplanets)=0.0D0
       planets%pincl(nplanets)=0.0D0
       planets%ecc(nplanets)=0.0D0
       planets%w(nplanets)=0.0D0
       planets%M(nplanets)=0.0D0
       planets%rp(nplanets)=0.0D0
       nplanets=nplanets-1
   ENDIF
 ENDIF
END SUBROUTINE dropplanet

SUBROUTINE load(simp,starp,planets,spots)
IMPLICIT NONE

INTEGER              :: status
TYPE(simparam_)      :: simp
TYPE(starparam_)     :: starp
TYPE(spots_)         :: spots
TYPE(planets_)       :: planets

CALL simload(simp)
IF (simp%nspots<0.OR.simp%nspots>MAXOBJECTS) call errmsg(3)
IF (simp%nplanets<0.OR.simp%nplanets>MAXOBJECTS) call errmsg(3)
IF ((simp%nspots+simp%nplanets<0).OR.(simp%nplanets>MAXOBJECTS)) call errmsg(3)
CALL loadstar(starp)

IF (simp%nspots>0) then
IF (ASSOCIATED(spots%id)) DEALLOCATE(spots%id) 
ALLOCATE (spots%id(simp%nspots),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(spots%lat)) DEALLOCATE(spots%lat) 
ALLOCATE (spots%lat(simp%nspots),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(spots%long)) DEALLOCATE(spots%long) 
ALLOCATE (spots%long(simp%nspots),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(spots%adim)) DEALLOCATE(spots%adim) 
ALLOCATE (spots%adim(simp%nspots),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(spots%ffact)) DEALLOCATE(spots%ffact) 
ALLOCATE (spots%ffact(simp%nspots),STAT=status)
IF (status/=0) call errmsg(27)
CALL loadspots(spots,simp%nspots)
ENDIF

IF (simp%nplanets>0) then
IF (ASSOCIATED(planets%id)) DEALLOCATE(planets%id)
ALLOCATE (planets%id(simp%nplanets),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%rp)) DEALLOCATE(planets%rp)
ALLOCATE (planets%rp(simp%nplanets),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%porb)) DEALLOCATE(planets%porb)
ALLOCATE (planets%porb(simp%nplanets),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%pincl)) DEALLOCATE(planets%pincl)
ALLOCATE (planets%pincl(simp%nplanets),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%ecc)) DEALLOCATE(planets%ecc)
ALLOCATE (planets%ecc(simp%nplanets),STAT=status)
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%w)) DEALLOCATE(planets%w)
ALLOCATE (planets%w(simp%nplanets))
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%omega)) DEALLOCATE(planets%omega)
ALLOCATE (planets%omega(simp%nplanets))
IF (status/=0) call errmsg(27)
IF (ASSOCIATED(planets%M)) DEALLOCATE(planets%M)
ALLOCATE (planets%M(simp%nplanets))
IF (status/=0) call errmsg(27)
CALL loadplanets(planets,simp%nplanets)
ENDIF

CALL check_inputs(simp,starp,planets,spots)

END SUBROUTINE load

SUBROUTINE unload(spots,planets,res)
IMPLICIT NONE

TYPE(spots_) :: spots
TYPE(planets_) :: planets
TYPE(res_)     :: res

! Deallocate memory for input vectors

IF (ASSOCIATED(spots%id)) DEALLOCATE (spots%id)
IF (ASSOCIATED(spots%lat)) DEALLOCATE (spots%lat)
IF (ASSOCIATED(spots%long)) DEALLOCATE (spots%long)
IF (ASSOCIATED(spots%adim)) DEALLOCATE (spots%adim)
IF (ASSOCIATED(spots%ffact)) DEALLOCATE (spots%ffact)

IF (ASSOCIATED(planets%id)) DEALLOCATE (planets%id)
IF (ASSOCIATED(planets%rp)) DEALLOCATE (planets%rp)
IF (ASSOCIATED(planets%porb)) DEALLOCATE (planets%porb)
IF (ASSOCIATED(planets%pincl)) DEALLOCATE (planets%pincl)
IF (ASSOCIATED(planets%ecc)) DEALLOCATE (planets%ecc)
IF (ASSOCIATED(planets%w)) DEALLOCATE (planets%w)
IF (ASSOCIATED(planets%omega)) DEALLOCATE (planets%omega)
IF (ASSOCIATED(planets%M)) DEALLOCATE (planets%M)

! Deallocate memory for result vectors

IF (ASSOCIATED(res%time)) DEALLOCATE(res%time)
IF (ASSOCIATED(res%flux)) DEALLOCATE(res%flux)
IF (ASSOCIATED(res%status)) DEALLOCATE(res%status)

END SUBROUTINE unload

SUBROUTINE ERRMSG(msgnum)
IMPLICIT NONE

INTEGER   :: msgnum
CHARACTER(LEN=80)  :: errormessage(100), MAX

WRITE(MAX,'(I10)') MAXOBJECTS
errormessage(1)='ERR 1: Error opening input file inputs.txt'
errormessage(2)='ERR 2: Error reading simulation parameters'
errormessage(3)='ERR 3: Invalid number of objects 0<=Nobj<=' // ADJUSTL(MAX)
errormessage(4)='ERR 4: Error reading star parameters'
errormessage(5)='ERR 5: Error reading spots parameters'
errormessage(6)='ERR 6: Error reading planets parameters'
errormessage(7)='ERR 7: Invalid number of iterations (nsteps)'
errormessage(8)='ERR 8: Invalid time step (timestep)'
errormessage(9)='ERR 9: Invalid stellar rotation period (prot)'
errormessage(10)='ERR 10: Invalid stellar inclination angle 0.<=i<=180.'
errormessage(11)='ERR 11: Invalid position angle of stellar axis 0.<=posang<=360.'
errormessage(12)='ERR 12: Invalid stellar density (rho)'
errormessage(13)='ERR 13: Invalid linear limb darkening coefficient c1 (0.<=c1<=1.)'
errormessage(14)='ERR 14: Invalid quadratic limb darkening coefficient c2 (0.<=c2<=1.)'            
errormessage(15)='ERR 15: Invalid spots contrast ratio (f<0 or 0<f<=1)'
errormessage(16)='ERR 16: Incorrect planetary orbital period (porb>0.)'
errormessage(17)='ERR 17: Incorrect planetary orbital inclination (-90<=pincl<=90.)'
errormessage(18)='ERR 18: Incorrect planetary eccentricity (0.<=ecc<1.)'
errormessage(19)='ERR 19: Incorrect planetary argument of periastron (0.<=w<=360.)'
errormessage(20)='ERR 20: Incorrect longitude of the ascending node (0.<=omega<=360.)'
errormessage(21)='ERR 21: Incorrect planetary initial mean anomaly (0.<=M<=360.)'
errormessage(22)='ERR 22: Incorrect planetary radius (rp>0.)'          
errormessage(23)='ERR 23: Incorrect spot latitude (-90.<=lat<=90.)'              
errormessage(24)='ERR 24: Incorrect spot longitude (0.<=long<=360.)'
errormessage(25)='ERR 25: Incorrect spot angular dimension (0.<adim<=180.)'
errormessage(26)='ERR 26: Overlapping regions with total contrast ratio > 1'
errormessage(27)='ERR 27: Not enough memory (load)'
errormessage(28)='ERR 28: Not enough memory (allocate_temporary_vectors)'
errormessage(29)='ERR 29: Not enough memory (filter)'
errormessage(30)='ERR 30: Not enough memory (intersect)'
errormessage(31)='ERR 31: Not enough memory (set_midangle)'
errormessage(32)='ERR 32: Not enough memory (check_tangent_point_star)'
errormessage(33)='ERR 33: Not enough memory (check_tangent_point_spots)'
errormessage(34)='ERR 34: Not enough memory (check_tangent_point_planets)'
errormessage(35)='ERR 35: Not enough memory (setroots)'
errormessage(36)='ERR 36: a<=1, invalid planetary semi-major axis'
errormessage(37)='ERR 37: USAGE: ./KSint -v niter'
errormessage(38)='ERR 38: Error in the roots calculation (INTERSECT)'
errormessage(39)='ERR 39: Error in the arc integration (INTEGRATE)'

IF (msgnum>=1.AND.msgnum<=39) THEN
  WRITE(*,'(A)') errormessage(msgnum)
  IF (msgnum<=37) THEN
  STOP
  ENDIF
ENDIF
END SUBROUTINE ERRMSG

END MODULE getInputs !==========================================================

!+
MODULE PolynomialRoots !========================================================
! ------------------------------------------------------------------------------
! PURPOSE - Solve for the roots of a polynomial equation with real
!   coefficients, up to quartic order. Retrun a code indicating the nature
!   of the roots found.
!
! AUTHORS - Alfred H. Morris, Naval Surface Weapons Center, Dahlgren,VA
!           William L. Davis, Naval Surface Weapons Center, Dahlgren,VA
!           Alan Miller,  CSIRO Mathematical & Information Sciences
!                         CLAYTON, VICTORIA, AUSTRALIA 3169
!                         http://www.mel.dms.csiro.au/~alan
!           Ralph L. Carmichael, Public Domain Aeronautical Software
!                         http://www.pdas.com
!     REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
!    ??    1.0 AHM&WLD Original coding                     
! 27Feb97  1.1   AM    Converted to be compatible with ELF90
! 12Jul98  1.2   RLC   Module format; numerous style changes
!  4Jan99  1.3   RLC   Made the tests for zero constant term exactly zero

  IMPLICIT NONE

  INTEGER,PARAMETER,PRIVATE:: SP=KIND(1.0), DP=KIND(1.0D0)
  REAL(DP),PARAMETER,PRIVATE:: ZERO=0.0D0, FOURTH=0.25D0, HALF=0.5D0
  REAL(DP),PARAMETER,PRIVATE:: ONE=1.0D0, TWO=2.0D0, THREE=3.0D0, FOUR=4.0D0
  COMPLEX(DP),PARAMETER,PRIVATE:: CZERO=(0.D0,0.D0)

  REAL(DP),PARAMETER,PRIVATE:: EPS=EPSILON(ONE)

  CHARACTER(LEN=*),PARAMETER,PUBLIC:: POLYROOTS_VERSION= "1.3 (4 Jan 1999)"
  INTEGER,PRIVATE:: outputCode
!    =0 degenerate equation
!    =1 one real root
!    =21 two identical real roots
!    =22 two distinct real roots
!    =23 two complex roots
!    =31 multiple real roots
!    =32 one real and two complex roots
!    =33 three distinct real roots
!    =41
!    =42 two real and two complex roots
!    =43
!    =44 four complex roots
  PRIVATE:: CubeRoot
  PUBLIC:: LinearRoot
  PRIVATE:: OneLargeTwoSmall
  PUBLIC:: QuadraticRoots
  PUBLIC:: CubicRoots
  PUBLIC:: QuarticRoots
  PUBLIC:: SolvePolynomial
!----------------------------------------------------------------------------
  INTERFACE Swap
    MODULE PROCEDURE SwapDouble, SwapSingle
  END INTERFACE

CONTAINS

!+
FUNCTION CubeRoot(x) RESULT(f)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the Cube Root of a REAL(DP) number. If the argument is
!   negative, then the cube root is also negative.
  REAL(DP),INTENT(IN) :: x
  REAL(DP):: f
!----------------------------------------------------------------------------
  IF (x < ZERO) THEN
    f=-EXP(LOG(-x)/THREE)
  ELSE IF (x > ZERO) THEN
    f=EXP(LOG(x)/THREE)
  ELSE
    f=ZERO
  END IF
  RETURN
END Function CubeRoot   ! ---------------------------------------------------

!+
SUBROUTINE LinearRoot(a, z)
! ---------------------------------------------------------------------------
! PURPOSE - COMPUTES THE ROOTS OF THE REAL POLYNOMIAL
!              A(1) + A(2)*Z 
!     AND STORES THE RESULTS IN Z. It is assumed that a(2) is non-zero.
  REAL(DP),INTENT(IN),DIMENSION(:):: a
  REAL(DP),INTENT(OUT):: z
!----------------------------------------------------------------------------
  IF (a(2)==0.0) THEN
    z=ZERO
  ELSE
    z=-a(1)/a(2)
  END IF
  RETURN
END Subroutine LinearRoot   ! -----------------------------------------------

!+
SUBROUTINE OneLargeTwoSmall(a1,a2,a4,w, z)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the roots of a cubic when one root, w, is known to be
!   much larger in magnitude than the other two

  REAL(DP),INTENT(IN):: a1,a2,a4
  REAL(DP),INTENT(IN):: w
  COMPLEX(DP),INTENT(OUT),DIMENSION(:):: z


  REAL(DP),DIMENSION(3):: aq
!----------------------------------------------------------------------------
  aq(1)=a1
  aq(2)=a2+a1/w
  aq(3)=-a4*w
  CALL QuadraticRoots(aq, z)
  z(3)=CMPLX(w,ZERO,DP)
  
  IF (AIMAG(z(1)) == ZERO) RETURN
  z(3)=z(2)
  z(2)=z(1)
  z(1)=CMPLX(w,ZERO,DP)
  RETURN
END Subroutine OneLargeTwoSmall   ! -----------------------------------------


!+
SUBROUTINE QuadraticRoots(a, z)
! ---------------------------------------------------------------------------
! PURPOSE - COMPUTES THE ROOTS OF THE REAL POLYNOMIAL
!              A(1) + A(2)*Z + A(3)*Z**2
!     AND STORES THE RESULTS IN Z.  IT IS ASSUMED THAT A(3) IS NONZERO.

  REAL(DP),INTENT(IN),DIMENSION(:):: a
  COMPLEX(DP),INTENT(OUT),DIMENSION(:):: z


  REAL(DP):: d, r, w, x, y
!----------------------------------------------------------------------------
  IF(a(1)==0.0) THEN     ! EPS is a global module constant (private)
    z(1) = CZERO               ! one root is obviously zero
    z(2) = CMPLX(-a(2)/a(3), ZERO,DP)    ! remainder is a linear eq.
    outputCode=21   ! two identical real roots
    RETURN
  END IF

  d = a(2)*a(2) - FOUR*a(1)*a(3)             ! the discriminant
  IF (ABS(d) <= TWO*eps*a(2)*a(2)) THEN
    z(1) = CMPLX(-HALF*a(2)/a(3), ZERO, DP) ! discriminant is tiny
    z(2) = z(1)
    outputCode=22  ! two distinct real roots
    RETURN
  END IF

  r = SQRT(ABS(d))
  IF (d < ZERO) THEN
    x = -HALF*a(2)/a(3)        ! negative discriminant => roots are complex   
    y = ABS(HALF*r/a(3))
    z(1) = CMPLX(x, y, DP)
    z(2) = CMPLX(x,-y, DP)    ! its conjugate
    outputCode=23             !  COMPLEX ROOTS
    RETURN
  END IF

  IF (a(2) /= ZERO) THEN              ! see Numerical Recipes, sec. 5.5
    w = -(a(2) + SIGN(r,a(2)))
    z(1) = CMPLX(TWO*a(1)/w,  ZERO, DP)
    z(2) = CMPLX(HALF*w/a(3), ZERO, DP)
    outputCode=22           ! two real roots
    RETURN
  END IF

  x = ABS(HALF*r/a(3))   ! a(2)=0 if you get here
  z(1) = CMPLX( x, ZERO, DP)
  z(2) = CMPLX(-x, ZERO, DP)
  outputCode=22
  RETURN
END Subroutine QuadraticRoots   ! -------------------------------------------


!+
SUBROUTINE CubicRoots(a, z)
!----------------------------------------------------------------------------
! PURPOSE - Compute the roots of the real polynomial
!              A(1) + A(2)*Z + A(3)*Z**2 + A(4)*Z**3
  REAL(DP),INTENT(IN),DIMENSION(:):: a
  COMPLEX(DP),INTENT(OUT),DIMENSION(:):: z

  REAL(DP),PARAMETER:: RT3=1.7320508075689D0    ! (Sqrt(3)
  REAL (DP) :: aq(3), arg, c, cf, d, p, p1, q, q1
  REAL(DP):: r, ra, rb, rq, rt
  REAL(DP):: r1, s, sf, sq, sum, t, tol, t1, w
  REAL(DP):: w1, w2, x, x1, x2, x3, y, y1, y2, y3

! NOTE -   It is assumed that a(4) is non-zero. No test is made here.
!----------------------------------------------------------------------------
  IF (a(1)==0.0) THEN
    z(1) = CZERO  ! one root is obviously zero
    CALL QuadraticRoots(a(2:4), z(2:3))   ! remaining 2 roots here
    RETURN
  END IF

  p = a(3)/(THREE*a(4))
  q = a(2)/a(4)
  r = a(1)/a(4)
  tol = FOUR*EPS

  c = ZERO
  t = a(2) - p*a(3)
  IF (ABS(t) > tol*ABS(a(2))) c = t/a(4)

  t = TWO*p*p - q
  IF (ABS(t) <= tol*ABS(q)) t = ZERO
  d = r + p*t
  IF (ABS(d) <= tol*ABS(r)) GO TO 110

!           SET  SQ = (A(4)/S)**2 * (C**3/27 + D**2/4)

  s = MAX(ABS(a(1)), ABS(a(2)), ABS(a(3)))
  p1 = a(3)/(THREE*s)
  q1 = a(2)/s
  r1 = a(1)/s

  t1 = q - 2.25D0*p*p
  IF (ABS(t1) <= tol*ABS(q)) t1 = ZERO
  w = FOURTH*r1*r1
  w1 = HALF*p1*r1*t
  w2 = q1*q1*t1/27.0D0

  IF (w1 >= ZERO) THEN
    w = w + w1
    sq = w + w2
  ELSE IF (w2 < ZERO) THEN
    sq = w + (w1 + w2)
  ELSE
    w = w + w2
    sq = w + w1
  END IF

  IF (ABS(sq) <= tol*w) sq = ZERO
  rq = ABS(s/a(4))*SQRT(ABS(sq))
  IF (sq >= ZERO) GO TO 40

!                   ALL ROOTS ARE REAL

  arg = ATAN2(rq, -HALF*d)
  cf = COS(arg/THREE)
  sf = SIN(arg/THREE)
  rt = SQRT(-c/THREE)
  y1 = TWO*rt*cf
  y2 = -rt*(cf + rt3*sf)
  y3 = -(d/y1)/y2

  x1 = y1 - p
  x2 = y2 - p
  x3 = y3 - p

  IF (ABS(x1) > ABS(x2)) CALL Swap(x1,x2)
  IF (ABS(x2) > ABS(x3)) CALL Swap(x2,x3)
  IF (ABS(x1) > ABS(x2)) CALL Swap(x1,x2)

  w = x3

  IF (ABS(x2) < 0.1D0*ABS(x3)) GO TO 70
  IF (ABS(x1) < 0.1D0*ABS(x2)) x1 = - (r/x3)/x2
  z(1) = CMPLX(x1, ZERO,DP)
  z(2) = CMPLX(x2, ZERO,DP)
  z(3) = CMPLX(x3, ZERO,DP)
  RETURN

!                  REAL AND COMPLEX ROOTS

40 ra =CubeRoot(-HALF*d - SIGN(rq,d))
  rb = -c/(THREE*ra)
  t = ra + rb
  w = -p
  x = -p
  IF (ABS(t) <= tol*ABS(ra)) GO TO 41
  w = t - p
  x = -HALF*t - p
  IF (ABS(x) <= tol*ABS(p)) x = ZERO
  41 t = ABS(ra - rb)
  y = HALF*rt3*t
  
  IF (t <= tol*ABS(ra)) GO TO 60
  IF (ABS(x) < ABS(y)) GO TO 50
  s = ABS(x)
  t = y/x
  GO TO 51
50 s = ABS(y)
  t = x/y
51 IF (s < 0.1D0*ABS(w)) GO TO 70
  w1 = w/s
  sum = ONE + t*t
  IF (w1*w1 < 0.01D0*sum) w = - ((r/sum)/s)/s
  z(1) = CMPLX(w, ZERO,DP)
  z(2) = CMPLX(x, y,DP)
  z(3) = CMPLX(x,-y,DP)
  RETURN

!               AT LEAST TWO ROOTS ARE EQUAL

60 IF (ABS(x) < ABS(w)) GO TO 61
  IF (ABS(w) < 0.1D0*ABS(x)) w = - (r/x)/x
  z(1) = CMPLX(w, ZERO,DP)
  z(2) = CMPLX(x, ZERO,DP)
  z(3) = z(2)
  RETURN
  61 IF (ABS(x) < 0.1D0*ABS(w)) GO TO 70
  z(1) = CMPLX(x, ZERO,DP)
  z(2) = z(1)
  z(3) = CMPLX(w, ZERO,DP)
  RETURN

!     HERE W IS MUCH LARGER IN MAGNITUDE THAN THE OTHER ROOTS.
!     AS A RESULT, THE OTHER ROOTS MAY BE EXCEEDINGLY INACCURATE
!     BECAUSE OF ROUNDOFF ERROR.  TO DEAL WITH THIS, A QUADRATIC
!     IS FORMED WHOSE ROOTS ARE THE SAME AS THE SMALLER ROOTS OF
!     THE CUBIC.  THIS QUADRATIC IS THEN SOLVED.

!     THIS CODE WAS WRITTEN BY WILLIAM L. DAVIS (NSWC).

70 aq(1) = a(1)
  aq(2) = a(2) + a(1)/w
  aq(3) = -a(4)*w
  CALL QuadraticRoots(aq, z)
  z(3) = CMPLX(w, ZERO,DP)
  
  IF (AIMAG(z(1)) == ZERO) RETURN
  z(3) = z(2)
  z(2) = z(1)
  z(1) = CMPLX(w, ZERO,DP)
  RETURN
!-----------------------------------------------------------------------


!                   CASE WHEN D = 0

110 z(1) = CMPLX(-p, ZERO,DP)
  w = SQRT(ABS(c))
  IF (c < ZERO) GO TO 120
  z(2) = CMPLX(-p, w,DP)
  z(3) = CMPLX(-p,-w,DP)
  RETURN

120 IF (p /= ZERO) GO TO 130
  z(2) = CMPLX(w, ZERO,DP)
  z(3) = CMPLX(-w, ZERO,DP)
  RETURN

130 x = -(p + SIGN(w,p))
  z(3) = CMPLX(x, ZERO,DP)
  t = THREE*a(1)/(a(3)*x)
  IF (ABS(p) > ABS(t)) GO TO 131
  z(2) = CMPLX(t, ZERO,DP)
  RETURN
131 z(2) = z(1)
  z(1) = CMPLX(t, ZERO,DP)
  RETURN
END Subroutine CubicRoots   ! -----------------------------------------------


!+
SUBROUTINE QuarticRoots(a,z)
!----------------------------------------------------------------------------
! PURPOSE - Compute the roots of the real polynomial
!               A(1) + A(2)*Z + ... + A(5)*Z**4

  REAL(DP), INTENT(IN)     :: a(:)
  COMPLEX(DP), INTENT(OUT) :: z(:)

  COMPLEX(DP) :: w
  REAL(DP):: b,b2, c, d, e, h, p, q, r, t
  REAL(DP),DIMENSION(4):: temp
  REAL(DP):: u, v, v1, v2, x, x1, x2, x3, y


! NOTE - It is assumed that a(5) is non-zero. No test is made here

!----------------------------------------------------------------------------

  IF (a(1)==0.0) THEN
    z(1) = CZERO    !  one root is obviously zero
    CALL CubicRoots(a(2:), z(2:))
    RETURN
  END IF


  b = a(4)/(FOUR*a(5))
  c = a(3)/a(5)
  d = a(2)/a(5)
  e = a(1)/a(5)
  b2 = b*b

  p = HALF*(c - 6.0D0*b2)
  q = d - TWO*b*(c - FOUR*b2)
  r = b2*(c - THREE*b2) - b*d + e

! SOLVE THE RESOLVENT CUBIC EQUATION. THE CUBIC HAS AT LEAST ONE
! NONNEGATIVE REAL ROOT.  IF W1, W2, W3 ARE THE ROOTS OF THE CUBIC
! THEN THE ROOTS OF THE ORIGINIAL EQUATION ARE
!     Z = -B + CSQRT(W1) + CSQRT(W2) + CSQRT(W3)
! WHERE THE SIGNS OF THE SQUARE ROOTS ARE CHOSEN SO
! THAT CSQRT(W1) * CSQRT(W2) * CSQRT(W3) = -Q/8.

  temp(1) = -q*q/64.0D0
  temp(2) = 0.25D0*(p*p - r)
  temp(3) =  p
  temp(4) = ONE
  CALL CubicRoots(temp,z)
  IF (AIMAG(z(2)) /= ZERO) GO TO 60

!         THE RESOLVENT CUBIC HAS ONLY REAL ROOTS
!         REORDER THE ROOTS IN INCREASING ORDER

  x1 = DBLE(z(1))
  x2 = DBLE(z(2))
  x3 = DBLE(z(3))
  IF (x1 > x2) CALL Swap(x1,x2)
  IF (x2 > x3) CALL Swap(x2,x3)
  IF (x1 > x2) CALL Swap(x1,x2)

  u = ZERO
  IF (x3 > ZERO) u = SQRT(x3)
  IF (x2 <= ZERO) GO TO 41
  IF (x1 >= ZERO) GO TO 30
  IF (ABS(x1) > x2) GO TO 40
  x1 = ZERO

30 x1 = SQRT(x1)
  x2 = SQRT(x2)
  IF (q > ZERO) x1 = -x1
  temp(1) = (( x1 + x2) + u) - b
  temp(2) = ((-x1 - x2) + u) - b
  temp(3) = (( x1 - x2) - u) - b
  temp(4) = ((-x1 + x2) - u) - b
  CALL SelectSort(temp)
  IF (ABS(temp(1)) >= 0.1D0*ABS(temp(4))) GO TO 31
  t = temp(2)*temp(3)*temp(4)
  IF (t /= ZERO) temp(1) = e/t
31 z(1) = CMPLX(temp(1), ZERO,DP)
  z(2) = CMPLX(temp(2), ZERO,DP)
  z(3) = CMPLX(temp(3), ZERO,DP)
  z(4) = CMPLX(temp(4), ZERO,DP)
  RETURN

40 v1 = SQRT(ABS(x1))
v2 = ZERO
GO TO 50
41 v1 = SQRT(ABS(x1))
v2 = SQRT(ABS(x2))
IF (q < ZERO) u = -u

50 x = -u - b
y = v1 - v2
z(1) = CMPLX(x, y,DP)
z(2) = CMPLX(x,-y,DP)
x =  u - b
y = v1 + v2
z(3) = CMPLX(x, y,DP)
z(4) = CMPLX(x,-y,DP)
RETURN

!                THE RESOLVENT CUBIC HAS COMPLEX ROOTS

60 t = DBLE(z(1))
x = ZERO
IF (t < ZERO) THEN
  GO TO 61
ELSE IF (t == ZERO) THEN
  GO TO 70
ELSE
  GO TO 62
END IF
61 h = ABS(DBLE(z(2))) + ABS(AIMAG(z(2)))
IF (ABS(t) <= h) GO TO 70
GO TO 80
62 x = SQRT(t)
IF (q > ZERO) x = -x

70 w = SQRT(z(2))
  u = TWO*DBLE(w)
  v = TWO*ABS(AIMAG(w))
  t =  x - b
  x1 = t + u
  x2 = t - u
  IF (ABS(x1) <= ABS(x2)) GO TO 71
  t = x1
  x1 = x2
  x2 = t
71 u = -x - b
  h = u*u + v*v
  IF (x1*x1 < 0.01D0*MIN(x2*x2,h)) x1 = e/(x2*h)
  z(1) = CMPLX(x1, ZERO,DP)
  z(2) = CMPLX(x2, ZERO,DP)
  z(3) = CMPLX(u, v,DP)
  z(4) = CMPLX(u,-v,DP)
  RETURN

80 v = SQRT(ABS(t))
  z(1) = CMPLX(-b, v,DP)
  z(2) = CMPLX(-b,-v,DP)
  z(3) = z(1)
  z(4) = z(2)
  RETURN

END Subroutine QuarticRoots


!+
SUBROUTINE SelectSort(a)
! ---------------------------------------------------------------------------
! PURPOSE - Reorder the elements of in increasing order.
  REAL(DP),INTENT(IN OUT),DIMENSION(:):: a

  INTEGER:: j
  INTEGER,DIMENSION(1):: k
! NOTE - This is a n**2 method. It should only be used for small arrays. <25
!----------------------------------------------------------------------------
  DO j=1,SIZE(a)-1
    k=MINLOC(a(j:))
    IF (j /= k(1)) CALL Swap(a(k(1)),a(j))
  END DO
  RETURN
END Subroutine SelectSort   ! -----------------------------------------------


!+
SUBROUTINE SolvePolynomial(quarticCoeff, cubicCoeff, quadraticCoeff, &
  linearCoeff, constantCoeff, code, root1,root2,root3,root4)
! ---------------------------------------------------------------------------
  REAL(DP),INTENT(IN):: quarticCoeff
  REAL(DP),INTENT(IN):: cubicCoeff, quadraticCoeff
  REAL(DP),INTENT(IN):: linearCoeff, constantCoeff
  INTEGER,INTENT(OUT):: code
  COMPLEX(DP),INTENT(OUT):: root1,root2,root3,root4
  REAL(DP),DIMENSION(5):: a
  COMPLEX(DP),DIMENSION(5):: z
!----------------------------------------------------------------------------
  a(1)=constantCoeff
  a(2)=linearCoeff
  a(3)=quadraticCoeff
  a(4)=cubicCoeff
  a(5)=quarticCoeff

  IF (quarticCoeff /= ZERO) THEN
    CALL QuarticRoots(a,z)  
  ELSE IF (cubicCoeff /= ZERO) THEN
    CALL CubicRoots(a,z)
  ELSE IF (quadraticCoeff /= ZERO) THEN
    CALL QuadraticRoots(a,z)
  ELSE IF (linearCoeff /= ZERO) THEN
    z(1)=CMPLX(-constantCoeff/linearCoeff, 0, DP)
    outputCode=1
  ELSE
    outputCode=0    !  { no roots }
  END IF

  code=outputCode
  IF (outputCode > 0) root1=z(1)
  IF (outputCode > 1) root2=z(2)
  IF (outputCode > 23) root3=z(3)
  IF (outputCode > 99) root4=z(4)
  RETURN
END Subroutine SolvePolynomial   ! ------------------------------------------

!+
SUBROUTINE SwapDouble(a,b)
! ---------------------------------------------------------------------------
! PURPOSE - Interchange the contents of a and b
  REAL(DP),INTENT(IN OUT):: a,b
  REAL(DP):: t
!----------------------------------------------------------------------------
  t=b
  b=a
  a=t
  RETURN
END Subroutine SwapDouble   ! -----------------------------------------------

!+
SUBROUTINE SwapSingle(a,b)
! ---------------------------------------------------------------------------
! PURPOSE - Interchange the contents of a and b
  REAL(SP),INTENT(IN OUT):: a,b
  REAL(SP):: t
!----------------------------------------------------------------------------
  t=b
  b=a
  a=t
  RETURN
END Subroutine SwapSingle   ! -----------------------------------------------


END Module PolynomialRoots   ! ==============================================

!+
MODULE Maths !===============================================================
! ---------------------------------------------------------------------------
! PURPOSE - Performs all the relevant mathematical operations to extract 
!           the photometric information. This includes further input checks, 
!           allocating/deallocating temporary arrays, projecting the objects 
!           into the observable plane, intersecting the objects, classifying 
!           the arcs, integrate using KS equations and optionally produce
!           an output arc table.
! 
! AUTHOR  - Marco Montalto, Centro de Astrofísica da Universidade do Porto,
!                           (CAUP)
!
!
!     REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
!    ??    1.0   MM    Original coding                     
!
USE getInputs
IMPLICIT NONE
PRIVATE

TYPE, PUBLIC                  :: startmp_
 INTEGER                      :: nroots=0
 INTEGER, POINTER             :: fgarc(:)=>null(), tgpoint(:)=>null()
 REAL(DP), POINTER            :: midangle(:)=>null()
 REAL(DP), POINTER            :: sumffact(:)=>null(), arcffact(:)=>null()
 REAL(DP)                     :: integral,integral_0
 REAL(DP), POINTER            :: roots(:)=>null() 
END TYPE startmp_

TYPE, PUBLIC                  :: spotstmp_
 INTEGER                      :: nspots=0
 INTEGER, POINTER             :: id(:) => null(),filter(:) => null()
 INTEGER, POINTER             :: nroots(:) => null(), fgarc(:,:)=>null(), tgpoint(:,:)=>null()
 REAL(DP), POINTER            :: y(:)=>null(), phi(:)=>null(), dim(:)=>null()
 REAL(DP), POINTER            :: midangle(:,:)=>null(), ffact(:)=>null()
 REAL(DP), POINTER            :: sumffact(:,:)=>null(), arcffact(:,:)=>null()
 REAL(DP), POINTER            :: roots(:,:)=>null()
 REAL(DP)                     :: integral
END TYPE spotstmp_

TYPE, PUBLIC                  :: planetstmp_
 INTEGER                      :: nplanets=0
 INTEGER, POINTER             :: id(:) => null(), filter(:) => null()
 INTEGER, POINTER             :: nroots(:) => null(), fgarc(:,:)=>null(), tgpoint(:,:)=>null()
 REAL(DP), POINTER            :: y(:)=>null(), phi(:)=>null(), dim(:)=>null()
 REAL(DP), POINTER            :: midangle(:,:)=>null()
 REAL(DP), POINTER            :: sumffact(:,:)=>null(), arcffact(:,:)=>null()
 REAL(DP), POINTER            :: roots(:,:)=>null()
 REAL(DP)                     :: integral
END TYPE planetstmp_

TYPE, PUBLIC                  :: plttmp_
 INTEGER, POINTER             :: fgarc(:,:)=>null(), nroots(:)=>null()
 REAL(DP), POINTER            :: arcffact(:,:)=>null(), roots(:,:)=>null()
END TYPE plttmp_

PUBLIC                        :: KSflux

INTERFACE GETZ
  MODULE PROCEDURE GETZ_PLANETS, GETZ_SPOTS
END INTERFACE GETZ

INTERFACE GETMET
  MODULE PROCEDURE GETMET_BORDER_SPOTS, GETMET_BORDER_PLANETS, & 
                   GETMET_PLANETS_PLANETS, GETMET_SPOTS_SPOTS, & 
                   GETMET_SPOTS_PLANETS, GETMET_PLANETS_SPOTS
END INTERFACE GETMET
 
CONTAINS

SUBROUTINE KSflux(simp,starp,planets,spots,res)
IMPLICIT NONE

INTEGER           :: iter, j, status
TYPE(spots_)      :: spots
TYPE(planets_)    :: planets
TYPE(simparam_)   :: simp
TYPE(starparam_)  :: starp
TYPE(startmp_)    :: startmp
TYPE(spotstmp_)   :: spotstmp
TYPE(planetstmp_) :: planetstmp
REAL(DP)          :: integral
TYPE(res_)        :: res

!
! Check all input parameters
!

CALL CHECK_INPUTS(simp,starp,planets,spots)

!
! Allocate memory for result vectors
!

IF (ASSOCIATED(res%time)) DEALLOCATE(res%time)
ALLOCATE(res%time(simp%nsteps))
IF (ASSOCIATED(res%flux)) DEALLOCATE(res%flux)
ALLOCATE(res%flux(simp%nsteps))
IF (ASSOCIATED(res%status)) DEALLOCATE(res%status)
ALLOCATE(res%status(simp%nsteps))

!
! If there are no objects we do not need to go further
!

IF ((simp%nplanets+simp%nspots)==0) THEN
  DO iter=1, simp%nsteps
     res%time(iter)=DBLE(iter)*simp%timestep
  ENDDO
     res%flux=1.0D0
     res%status=0
  RETURN
ENDIF

!
! Allocate memory
!

CALL ALLOCATE_TEMPORARY_VECTORS(simp,spotstmp,planetstmp)

DO iter=1, simp%nsteps

!
! Set iteration time
!

res%time(iter)=DBLE(iter)*simp%timestep
status=0

!
! Project objects on the observer plane
!

CALL PROJECT(iter,simp,starp,spots,planets,spotstmp,planetstmp)
IF ((planetstmp%nplanets+spotstmp%nspots)==0) THEN
  res%flux(iter)=1.0D0
  res%status(iter)=0
  CYCLE
ENDIF

!
! Intersect objects
!

CALL INTERSECT(iter,startmp,spotstmp,planetstmp,status)
IF (status/=0) THEN 
  res%flux(iter)=1.0D0
  res%status(iter)=status
  CALL errmsg(status)
  CYCLE
ENDIF

!
! Classify arcs
!

CALL CLASSIFYARCS(startmp,planetstmp,spotstmp)

!
! Integrate arcs
!

CALL INTEGRATE(starp,startmp,spotstmp,planetstmp,integral,status)
IF (status/=0) THEN 
  res%flux(iter)=1.0D0
  res%status(iter)=status
  CALL errmsg(status)
  CYCLE
ENDIF
res%flux(iter)=integral
res%status(iter)=status

!
! Optionally produce arc table
!

CALL PLOT(iter,simp,starp,startmp,planetstmp,spotstmp)

ENDDO

! Release memory

CALL DEALLOCATE_TEMPORARY_VECTORS(startmp,spotstmp,planetstmp)

END SUBROUTINE KSflux

SUBROUTINE ALLOCATE_TEMPORARY_VECTORS(simp,spotstmp,planetstmp)
IMPLICIT NONE

INTEGER :: i, status
TYPE(simparam_)   ::  simp
TYPE(spotstmp_)  :: spotstmp
TYPE(planetstmp_)  :: planetstmp

IF (simp%nspots>0) THEN
IF (ASSOCIATED(spotstmp%id)) DEALLOCATE (spotstmp%id)
ALLOCATE (spotstmp%id(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(spotstmp%y)) DEALLOCATE (spotstmp%y)
ALLOCATE (spotstmp%y(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(spotstmp%phi)) DEALLOCATE (spotstmp%phi)
ALLOCATE (spotstmp%phi(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(spotstmp%filter)) DEALLOCATE (spotstmp%filter)
ALLOCATE (spotstmp%filter(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(spotstmp%dim)) DEALLOCATE (spotstmp%dim)
ALLOCATE (spotstmp%dim(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(spotstmp%ffact)) DEALLOCATE (spotstmp%ffact)
ALLOCATE (spotstmp%ffact(simp%nspots), STAT=status)
IF (status/=0) call errmsg(28)
ENDIF
spotstmp%nspots=simp%nspots

IF (simp%nplanets>0) THEN
IF (ASSOCIATED(planetstmp%id)) DEALLOCATE (planetstmp%id)
ALLOCATE (planetstmp%id(simp%nplanets), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(planetstmp%y)) DEALLOCATE (planetstmp%y)
ALLOCATE (planetstmp%y(simp%nplanets), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(planetstmp%phi)) DEALLOCATE (planetstmp%phi)
ALLOCATE (planetstmp%phi(simp%nplanets), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(planetstmp%filter)) DEALLOCATE (planetstmp%filter)
ALLOCATE (planetstmp%filter(simp%nplanets), STAT=status)
IF (status/=0) call errmsg(28)
IF (ASSOCIATED(planetstmp%dim)) DEALLOCATE (planetstmp%dim)
ALLOCATE (planetstmp%dim(simp%nplanets), STAT=status)
IF (status/=0) call errmsg(28)
ENDIF
planetstmp%nplanets=simp%nplanets

END SUBROUTINE ALLOCATE_TEMPORARY_VECTORS

SUBROUTINE PROJECT(iter,simp,starp,spots,planets,spotstmp,planetstmp)   
USE getInputs
IMPLICIT NONE

INTEGER           :: i,iter
TYPE(spots_)      :: spots
TYPE(planets_)    :: planets
TYPE(spotstmp_)   :: spotstmp
TYPE(planetstmp_) :: planetstmp
TYPE(simparam_)   :: simp
TYPE(starparam_)  :: starp
REAL(DP)          :: T0, dT, prot, incl
REAL(DP)          :: posang, rho
REAL(DP)          :: lat, long0, long, x, y, z
REAL(DP)          :: period,ecc,inclination
REAL(DP)          :: argperi,M0,M,a,E,nu,r,omega

dT=simp%timestep 
prot=starp%prot
incl=rad(starp%incl)
posang=setangle(rad(starp%posang)-HPI)
rho=starp%rho
T0=0.0d0

spotstmp%nspots=simp%nspots
planetstmp%nplanets=simp%nplanets

IF (spotstmp%nspots>0) THEN
spotstmp%filter=0
DO i=1, simp%nspots
   spotstmp%id(i)=spots%id(i)
   spotstmp%y(i)=0.0D0
   spotstmp%phi(i)=0.0D0
   spotstmp%filter(i)=0
   spotstmp%dim(i)=rad(spots%adim(i))
   spotstmp%ffact(i)=spots%ffact(i)
ENDDO
ENDIF

IF (planetstmp%nplanets>0) THEN
planetstmp%filter=0
DO i=1, simp%nplanets
   planetstmp%id(i)=planets%id(i)
   planetstmp%y(i)=0.0D0
   planetstmp%phi(i)=0.0D0
   planetstmp%filter(i)=0
   planetstmp%dim(i)=planets%rp(i)
ENDDO
ENDIF


IF (spotstmp%nspots>0) THEN
DO i=1, spotstmp%nspots

 lat=rad(spots%lat(i))
 long0=setangle(rad(spots%long(i))-HPI)
 long=long0+(T0+dT*real(iter)) / prot * TWPI - real(int((dT*real(iter)) / prot)) * TWPI

 x=cos(posang)*(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))+cos(lat)*sin(long)*sin(posang) 
 y=cos(lat)*sin(long)*cos(posang)-(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))*sin(posang)
 z=sin(lat)*cos(incl)-cos(lat)*cos(long)*sin(incl)

if (z.ne.0.0D0) then
 if ((x.ne.0.0D0).or.(y.ne.0.0D0)) then
     spotstmp%y(i)=HPI-atan2(z,sqrt(x**2+y**2))
 else
   if (z.gt.0.0D0) then
     spotstmp%y(i)=0.0D0
   else
     spotstmp%y(i)=PI
   endif
 endif
else
     spotstmp%y(i)=HPI
endif
               
if (x/=0.0D0) then
   if (y/=0.0D0) then
       spotstmp%phi(i)=atan2(y,x)-HPI
   else
     if (x>0.0D0) then
       spotstmp%phi(i)=-HPI
     else
       spotstmp%phi(i)=HPI
     endif
   endif
else
   if (y>0.0D0) then
       spotstmp%phi(i)=0.0D0
   else if (y<0.) then
       spotstmp%phi(i)=PI
   else
       spotstmp%phi(i)=0.0D0
   endif
endif

IF (sin(spotstmp%y(i))<epsilon) THEN
       spotstmp%y(i)=0.0D0
       spotstmp%phi(i)=0.0D0
ENDIF


CALL SET_ANGLE(spotstmp%phi(i))
CALL SET_ANGLE(spotstmp%y(i))

ENDDO
ENDIF

IF (simp%nplanets>0) THEN
DO i=1, simp%nplanets

   period=planets%porb(i)      
   ecc=planets%ecc(i)
   inclination=rad(planets%pincl(i))
   argperi=rad(planets%w(i))
   M0=rad(planets%M(i))
   omega=rad(planets%omega(i))

!
!  Period in days, density in g/cm^3
!
   a=(3.75226985055D0)*((period)**(2./3.))*((rho)**(1./3.))
   IF (a<=1.0D0) THEN
      CALL ERRMSG(36)
      RETURN
   ENDIF
   M=M0+(TWPI/period)*(dT*real(iter))
   CALL SOLVE_KEPLER_EQUATION(M,ecc,E)           
   nu=2.*atan2(sqrt(1.+ecc)*sin(E/2.),sqrt(1.-ecc)*cos(E/2.))
   r=a*(1.-ecc**2)/(1.+ecc*cos(nu))          

   x=r*(cos(omega)*cos(nu+argperi)-sin(omega)*sin(nu+argperi)*cos(inclination))
   y=r*(sin(omega)*cos(nu+argperi)+cos(omega)*sin(nu+argperi)*cos(inclination))
   z=r*sin(nu+argperi)*sin(inclination)


!  Flag planets passing behind the star
   if (z.lt.0.0D0) planetstmp%filter(i)=1

   planetstmp%phi(i)=atan2(y,x)-HPI
   planetstmp%y(i)=sqrt(x**2+y**2)

   IF (planetstmp%y(i)<epsilon) THEN
      planetstmp%phi(i)=0.0d0
      planetstmp%y(i)=0.0d0
   ENDIF

   CALL SET_ANGLE(planetstmp%phi(i))
ENDDO
ENDIF

CALL FILTER(spotstmp,planetstmp)

END SUBROUTINE PROJECT

SUBROUTINE FILTER(spotstmp,planetstmp)
IMPLICIT NONE

INTEGER              :: i, j, status
TYPE(spotstmp_)      :: spotstmp, spotscp
TYPE(planetstmp_)    :: planetstmp, planetscp

!
! This routine filters the lists of objects flagging those planets that 
! are outside the star (or planets which projections at this timestep are
! coincident) or those spots that are in the invisible hemisphere 
! as seen by the observer. Coincident spots were already 
! considered at the beginning.
!

IF (spotstmp%nspots>0.) THEN

       DO i=1, spotstmp%nspots
         IF (spotstmp%filter(i)/=1) THEN
           IF (spotstmp%y(i)>(HPI+spotstmp%dim(i))) THEN
               spotstmp%filter(i)=1
           ELSE
               spotstmp%filter(i)=0
           ENDIF
         ENDIF
       ENDDO

!
!   Now drop those objects that have been filtered out and update the 
!   object lists.
!

IF (ASSOCIATED(spotscp%id)) DEALLOCATE(spotscp%id)
IF (ASSOCIATED(spotscp%y)) DEALLOCATE(spotscp%y)
IF (ASSOCIATED(spotscp%phi)) DEALLOCATE(spotscp%phi)
IF (ASSOCIATED(spotscp%filter)) DEALLOCATE(spotscp%filter)
IF (ASSOCIATED(spotscp%ffact)) DEALLOCATE(spotscp%ffact)
IF (ASSOCIATED(spotscp%dim)) DEALLOCATE(spotscp%dim)

ALLOCATE (spotscp%id(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (spotscp%phi(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (spotscp%dim(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (spotscp%y(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (spotscp%ffact(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (spotscp%filter(spotstmp%nspots), STAT=status)
IF (status/=0) call errmsg(29)
spotscp%id=0
spotscp%y=0.0D0
spotscp%phi=0.0D0
spotscp%ffact=0.0D0
spotscp%filter=0
spotscp%nspots=0

j=0
DO i=1, spotstmp%nspots
   IF (spotstmp%filter(i)==0) THEN
       j=j+1
       spotscp%id(j)=spotstmp%id(i)
       spotscp%y(j)=spotstmp%y(i)
       spotscp%phi(j)=spotstmp%phi(i)
       spotscp%ffact(j)=spotstmp%ffact(i)
       spotscp%filter(j)=spotstmp%filter(i)
       spotscp%dim(j)=spotstmp%dim(i)
   ENDIF
ENDDO
spotscp%nspots=j

spotstmp%id=spotscp%id
spotstmp%y=spotscp%y
spotstmp%phi=spotscp%phi
spotstmp%filter=0
spotstmp%ffact=spotscp%ffact
spotstmp%dim=spotscp%dim
spotstmp%nspots=spotscp%nspots

IF (ASSOCIATED(spotscp%id)) DEALLOCATE(spotscp%id)
IF (ASSOCIATED(spotscp%y)) DEALLOCATE(spotscp%y)
IF (ASSOCIATED(spotscp%phi)) DEALLOCATE(spotscp%phi)
IF (ASSOCIATED(spotscp%filter)) DEALLOCATE(spotscp%filter)
IF (ASSOCIATED(spotscp%ffact)) DEALLOCATE(spotscp%ffact)
IF (ASSOCIATED(spotscp%dim)) DEALLOCATE(spotscp%dim)

spotscp%nspots=0

ENDIF

IF (planetstmp%nplanets>0) THEN

       DO i=1, planetstmp%nplanets

          IF (planetstmp%filter(i)/=1) THEN

            IF (planetstmp%y(i)>(1.+planetstmp%dim(i))) THEN
              planetstmp%filter(i)=1              
            ELSE
              planetstmp%filter(i)=0
            ENDIF

            IF (planetstmp%nplanets>1) THEN
            DO j=i+1, planetstmp%nplanets
                 IF (j/=i) THEN
                   IF ((ABS(planetstmp%y(j)-planetstmp%y(i))<epsilon).AND. & 
                       (ABS(planetstmp%dim(j)-planetstmp%dim(i))<epsilon).AND. & 
                       (ABS(planetstmp%phi(i)-planetstmp%phi(j))<epsilon)) THEN
                        planetstmp%filter(j)=1
                   ENDIF   
                 ENDIF
            ENDDO
            ENDIF

          ENDIF

       ENDDO

!
!   Now drop those objects that have been filtered out and update the 
!   object lists.
!

IF (ASSOCIATED(planetscp%id)) DEALLOCATE(planetscp%id)
IF (ASSOCIATED(planetscp%y)) DEALLOCATE(planetscp%y)
IF (ASSOCIATED(planetscp%phi)) DEALLOCATE(planetscp%phi)
IF (ASSOCIATED(planetscp%filter)) DEALLOCATE(planetscp%filter)
IF (ASSOCIATED(planetscp%dim)) DEALLOCATE(planetscp%dim)

ALLOCATE (planetscp%id(planetstmp%nplanets), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (planetscp%phi(planetstmp%nplanets), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (planetscp%dim(planetstmp%nplanets), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (planetscp%y(planetstmp%nplanets), STAT=status)
IF (status/=0) call errmsg(29)
ALLOCATE (planetscp%filter(planetstmp%nplanets), STAT=status)
IF (status/=0) call errmsg(29)
planetscp%id=0
planetscp%y=0.0D0
planetscp%phi=0.0D0
planetscp%filter=0
planetscp%nplanets=0

j=0
DO i=1, planetstmp%nplanets
   IF (planetstmp%filter(i)==0) THEN
       j=j+1
       planetscp%id(j)=planetstmp%id(i)
       planetscp%y(j)=planetstmp%y(i)
       planetscp%phi(j)=planetstmp%phi(i)
       planetscp%filter(j)=planetstmp%filter(i)
       planetscp%dim(j)=planetstmp%dim(i)
   ENDIF
ENDDO
planetscp%nplanets=j

planetstmp%id=planetscp%id
planetstmp%y=planetscp%y
planetstmp%phi=planetscp%phi
planetstmp%filter=0
planetstmp%nplanets=planetscp%nplanets

IF (ASSOCIATED(planetscp%id)) DEALLOCATE(planetscp%id)
IF (ASSOCIATED(planetscp%y)) DEALLOCATE(planetscp%y)
IF (ASSOCIATED(planetscp%phi)) DEALLOCATE(planetscp%phi)
IF (ASSOCIATED(planetscp%filter)) DEALLOCATE(planetscp%filter)
IF (ASSOCIATED(planetscp%dim)) DEALLOCATE(planetscp%dim)
planetscp%nplanets=0

ENDIF

END SUBROUTINE FILTER

SUBROUTINE INTERSECT(iter,startmp,spotstmp,planetstmp,status)
IMPLICIT NONE

INTEGER, PARAMETER  :: newrapsiter=1
INTEGER :: i, j, k, nrpl, nrpl1, nrpl2, nrsp, nrsp1, nrsp2, nrst, nriter, Nsol, iter, status
REAL(DP)    :: y1, dim1, phi1, phi, beta_, psi_, theta_, xx, yy, xx_, yy_, xx_s, yy_s
REAL(DP)    :: y2, dim2, phi2, dist, alpha, alpha1, x0, y0
REAL(DP)    :: cos_rho, sin_rho, delta1, delta2, gamma1, gamma2, cos_rho1, cos_rho2
REAL(DP)    :: sin_rho1, sin_rho2, dr1dg2, dr2dg1, chi1, chi2  
REAL(DP)    :: rho, drho_dalpha, cot1, cot2, root_,cos_alpha,sin_alpha,theta2
REAL(DP) :: roots(4)
TYPE(planetstmp_)  :: planetstmp
TYPE(spotstmp_)    :: spotstmp
TYPE(startmp_)     :: startmp

status=0

IF (planetstmp%nplanets>0) THEN
 IF (ASSOCIATED(planetstmp%roots)) DEALLOCATE (planetstmp%roots)
 IF (ASSOCIATED(planetstmp%fgarc)) DEALLOCATE (planetstmp%fgarc)
 IF (ASSOCIATED(planetstmp%nroots)) DEALLOCATE (planetstmp%nroots)
 IF (ASSOCIATED(planetstmp%midangle)) DEALLOCATE (planetstmp%midangle)
 IF (ASSOCIATED(planetstmp%sumffact)) DEALLOCATE (planetstmp%sumffact)
 IF (ASSOCIATED(planetstmp%arcffact)) DEALLOCATE (planetstmp%arcffact)
 ALLOCATE (planetstmp%roots(planetstmp%nplanets, &
          (planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (planetstmp%fgarc(planetstmp%nplanets,&
          (planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (planetstmp%midangle(planetstmp%nplanets,&
          (planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (planetstmp%sumffact(planetstmp%nplanets,&
          (planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (planetstmp%arcffact(planetstmp%nplanets,&
          (planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (planetstmp%nroots(planetstmp%nplanets), STAT=status)
 IF (status/=0) call errmsg(30)
 planetstmp%roots=0.0D0
 planetstmp%sumffact=0.0D0
 planetstmp%arcffact=0.0D0
 planetstmp%midangle=0.0D0
 planetstmp%fgarc=0
 planetstmp%nroots=0
 ENDIF

 IF (spotstmp%nspots>0) THEN
 IF (ASSOCIATED(spotstmp%roots)) DEALLOCATE (spotstmp%roots)
 IF (ASSOCIATED(spotstmp%fgarc)) DEALLOCATE (spotstmp%fgarc)
 IF (ASSOCIATED(spotstmp%nroots)) DEALLOCATE (spotstmp%nroots)
 IF (ASSOCIATED(spotstmp%midangle)) DEALLOCATE (spotstmp%midangle)
 IF (ASSOCIATED(spotstmp%sumffact)) DEALLOCATE (spotstmp%sumffact)
 IF (ASSOCIATED(spotstmp%arcffact)) DEALLOCATE (spotstmp%arcffact)
 ALLOCATE (spotstmp%roots(spotstmp%nspots,&
           (planetstmp%nplanets)*4+(spotstmp%nspots-1)*2+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (spotstmp%fgarc(spotstmp%nspots,&
           planetstmp%nplanets*4+(spotstmp%nspots-1)*2+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (spotstmp%midangle(spotstmp%nspots,&
           planetstmp%nplanets*4+(spotstmp%nspots-1)*2+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (spotstmp%sumffact(spotstmp%nspots,&
           planetstmp%nplanets*4+(spotstmp%nspots-1)*2+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (spotstmp%arcffact(spotstmp%nspots,&
           planetstmp%nplanets*4+(spotstmp%nspots-1)*2+2), STAT=status)
 IF (status/=0) call errmsg(30)
 ALLOCATE (spotstmp%nroots(spotstmp%nspots), STAT=status)
 IF (status/=0) call errmsg(30)
 spotstmp%roots=0.0D0
 spotstmp%sumffact=0.0D0
 spotstmp%arcffact=0.0D0
 spotstmp%midangle=0.0D0
 spotstmp%fgarc=0
 spotstmp%nroots=0
 ENDIF

IF (ASSOCIATED(startmp%roots)) DEALLOCATE (startmp%roots)
IF (ASSOCIATED(startmp%fgarc)) DEALLOCATE (startmp%fgarc)
IF (ASSOCIATED(startmp%midangle)) DEALLOCATE (startmp%midangle)
IF (ASSOCIATED(startmp%sumffact)) DEALLOCATE (startmp%sumffact)
IF (ASSOCIATED(startmp%arcffact)) DEALLOCATE (startmp%arcffact)
ALLOCATE (startmp%roots(planetstmp%nplanets*2+spotstmp%nspots*2), STAT=status)
IF (status/=0) call errmsg(30)
ALLOCATE (startmp%fgarc(planetstmp%nplanets*2+spotstmp%nspots*2), STAT=status)
IF (status/=0) call errmsg(30)
ALLOCATE (startmp%midangle(planetstmp%nplanets*2+spotstmp%nspots*2), STAT=status)
IF (status/=0) call errmsg(30)
ALLOCATE (startmp%sumffact(planetstmp%nplanets*2+spotstmp%nspots*2), STAT=status)
IF (status/=0) call errmsg(30)
ALLOCATE (startmp%arcffact(planetstmp%nplanets*2+spotstmp%nspots*2), STAT=status)
IF (status/=0) call errmsg(30)
startmp%roots=0.0D0
startmp%fgarc=0.0D0
startmp%sumffact=0.0D0
startmp%midangle=0.0D0
startmp%arcffact=0.0D0
startmp%nroots=0

!
! Planets interception with the stellar border
!
IF (planetstmp%nplanets>0) THEN
DO i=1, planetstmp%nplanets

 y1=planetstmp%y(i)
 dim1=planetstmp%dim(i)
 phi=planetstmp%phi(i)
 nrpl=planetstmp%nroots(i)
 nrst=startmp%nroots

 IF (y1>epsilon.OR.(ABS(dim1-1.)>epsilon)) THEN     ! Object not perfectly coincident with the stellar border

  IF ((y1>(1.-dim1)).AND.(y1<(1.+dim1))) THEN       ! Two interception points

              psi_=acos((y1**2+dim1**2-1.)/(2.*y1*dim1))

!  
!       First root
!
              nrpl=nrpl+1
              nrst=nrst+1

              beta_=1.5*PI-psi_
              xx_=dim1*cos(beta_)
              yy_=dim1*sin(beta_)+y1
              call BACKWARD(xx_,yy_,xx,yy,phi)
              startmp%roots(nrst)=setangle(atan2(yy,xx))
              startmp%nroots=nrst

              planetstmp%roots(i,nrpl)=setangle(beta_)
              planetstmp%nroots(i)=nrpl

!  
!       Second root
!
              nrpl=nrpl+1
              nrst=nrst+1              

              beta_=psi_-HPI
              xx_=dim1*cos(beta_)
              yy_=dim1*sin(beta_)+y1
              call BACKWARD(xx_,yy_,xx,yy,phi)
              startmp%roots(nrst)=setangle(atan2(yy,xx))
              startmp%nroots=nrst

              planetstmp%roots(i,nrpl)=setangle(beta_)
              planetstmp%nroots(i)=nrpl

  ELSE IF (y1.EQ.(1.-dim1)) THEN ! Internally tangent

              nrpl=nrpl+1
              nrst=nrst+1

              beta_=HPI
              xx_=dim1*cos(beta_)
              yy_=dim1*sin(beta_)+y1
              call BACKWARD(xx_,yy_,xx,yy,phi)

              startmp%roots(nrst)=setangle(atan2(yy,xx))
              startmp%nroots=nrst

              planetstmp%roots(i,nrpl)=setangle(beta_)
              planetstmp%nroots(i)=nrpl

  ELSE IF (y1.EQ.(1.+dim1)) THEN ! Externally tangent

              nrpl=nrpl+1
              nrst=nrst+1

              beta_=1.5*PI
              xx_=dim1*cos(beta_)
              yy_=dim1*sin(beta_)+y1
              call BACKWARD(xx_,yy_,xx,yy,phi)

              startmp%roots(nrst)=setangle(atan2(yy,xx))
              startmp%nroots=nrst

              planetstmp%roots(i,nrpl)=setangle(beta_)
              planetstmp%nroots(i)=nrpl

  ENDIF
 ENDIF

ENDDO
ENDIF

!
! Spots interception with the stellar border
!
IF (spotstmp%nspots>0) THEN
DO i=1, spotstmp%nspots

   y1=spotstmp%y(i)
   dim1=spotstmp%dim(i)
   phi1=spotstmp%phi(i)
   nrsp1=spotstmp%nroots(i)
   nrst=startmp%nroots

   IF ((dim1>=HPI).AND.(y1<=HPI).AND.(dim1>=HPI+y1)) CYCLE
   IF ((dim1>=HPI).AND.(y1>=HPI).AND.(dim1>=1.50D0*PI-y1)) CYCLE
   IF ((ABS(dim1-HPI)<epsilon).AND.((ABS(y1-PI)<epsilon).OR.(ABS(y1)<epsilon))) CYCLE ! Half hemispheric spot totally invisible/visible

           if ((y1>(HPI-dim1)).and. &
               (y1<(HPI+dim1))) then

                 cot1=1./tan(y1)
                 cot2=1./tan(dim1)

          !             
          ! First root
          !
                 nrsp1=nrsp1+1
                 nrst=nrst+1

                 xx_=sin(dim1)*sqrt(1.-(cot1*cot2)**2)
                 yy_=sin(y1)*cos(dim1)-cos(y1)* &
                     sin(dim1)*(-cot1*cot2)

                 call backward(xx_,yy_,xx,yy,phi1)

                 if (yy.ge.0.0D0) then
                     startmp%roots(nrst)=setangle(acos(xx/ &
                              sqrt((xx)**2+(yy)**2))) 
                 else
                     startmp%roots(nrst)=setangle(TWPI-acos(xx/ &
                              sqrt((xx)**2+(yy)**2)))
                 endif   
                 startmp%nroots=nrst

                 spotstmp%roots(i,nrsp1)=setangle(acos(-cot1*cot2))
                 spotstmp%nroots(i)=nrsp1


          !
          ! Second root
          !
                 nrsp1=nrsp1+1
                 nrst=nrst+1

                 xx_=-sin(dim1)*sqrt(1.-(cot1*cot2)**2)
                 yy_=sin(y1)*cos(dim1)-cos(y1)* &
                     sin(dim1)*(-cot1*cot2)

                 call backward(xx_,yy_,xx,yy,phi1)

                 if (yy.ge.0.0D0) then
                    startmp%roots(nrst)=setangle(acos(xx/ &
                              sqrt((xx)**2+(yy)**2))) 
                 else
                    startmp%roots(nrst)=setangle(TWPI-acos(xx/ &
                              sqrt((xx)**2+(yy)**2)))
                 endif   
                 startmp%nroots=nrst

                 spotstmp%roots(i,nrsp1)=setangle(TWPI-acos(-cot1*cot2))
                 spotstmp%nroots(i)=nrsp1

            else if (y1==(HPI-dim1)) then

               xx_=0.0D0
               yy_=1.0D0
               call backward(xx_,yy_,xx,yy,phi)

               nrsp1=nrsp1+1
               nrst=nrst+1

                 if (yy>=0.0D0) then
                    startmp%roots(nrst)=setangle(acos(xx/ &
                              sqrt((xx)**2+(yy)**2))) 
                 else
                    startmp%roots(nrst)=setangle(TWPI-acos(xx/ &
                              sqrt((xx)**2+(yy)**2)))
                 endif   
                 startmp%nroots=nrst

                 spotstmp%roots(i,nrsp1)=setangle(TWPI-acos(-cot1*cot2))
                 spotstmp%nroots(i)=nrsp1
 
            else if (y1==(HPI+dim1)) then

               xx_=0.0D0
               yy_=1.0D0
               call backward(xx_,yy_,xx,yy,phi)

               nrsp1=nrsp1+1
               nrst=nrst+1

                 if (yy>=0.0D0) then
                    startmp%roots(nrst)=setangle(acos(xx/ &
                              sqrt((xx)**2+(yy)**2)))
                 else
                    startmp%roots(nrst)=setangle(TWPI-acos(xx/ &
                              sqrt((xx)**2+(yy)**2)))
                 endif   
                 startmp%nroots=nrst

                 spotstmp%roots(i,nrsp1)=setangle(TWPI-acos(-cot1*cot2))
                 spotstmp%nroots(i)=nrsp1

             endif

ENDDO
ENDIF

!
!        Two planets interception
!
IF (planetstmp%nplanets>1) THEN
 DO i=1, planetstmp%nplanets
  DO j=i+1, planetstmp%nplanets


   y1=planetstmp%y(i)
   dim1=planetstmp%dim(i)
   phi1=planetstmp%phi(i)
   nrpl1=planetstmp%nroots(i)
   y2=planetstmp%y(j)
   dim2=planetstmp%dim(j)
   phi2=planetstmp%phi(j)
   nrpl2=planetstmp%nroots(j)
   phi=phi2-phi1

   IF ((y1/=y2).OR.(dim1/=dim2)) THEN

   dist=sqrt(y1**2+y2**2-2.*y1*y2*cos(phi))
   IF (dist<(dim1+dim2).AND.dist>ABS(dim1-dim2)) THEN
   alpha1=acos((dim1**2+dist**2-dim2**2)/(2.0d0*dim1*dist))
  
   xx_=y2*cos(phi2+HPI)-y1*cos(phi1+HPI)
   yy_=y2*sin(phi2+HPI)-y1*sin(phi1+HPI)
   alpha=atan2(yy_,xx_)


!
!        First root
!
         nrpl1=nrpl1+1
         nrpl2=nrpl2+1

         xx_s=dim1*cos(alpha-alpha1)+y1*cos(phi1+HPI)
         yy_s=dim1*sin(alpha-alpha1)+y1*sin(phi1+HPI)

         call forward(xx_s,yy_s,xx,yy,phi1)

         planetstmp%roots(i,nrpl1)=setangle(atan2(yy-y1,xx))
         planetstmp%nroots(i)=nrpl1

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl2)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl2


!
!        Second root
!
         nrpl1=nrpl1+1
         nrpl2=nrpl2+1

         xx_s=dim1*cos(alpha+alpha1)+y1*cos(phi1+HPI)
         yy_s=dim1*sin(alpha+alpha1)+y1*sin(phi1+HPI)

         call forward(xx_s,yy_s,xx,yy,phi1)

         planetstmp%roots(i,nrpl1)=setangle(atan2(yy-y1,xx))
         planetstmp%nroots(i)=nrpl1

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl2)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl2

  ELSE IF (dist==(dim1+dim2).OR.(dist==ABS(dim1-dim2).AND.(dim1/=dim2))) THEN

   alpha1=acos((dim1**2+dist**2-dim2**2)/(2.0d0*dim1*dist))
  
   xx_=y2*cos(phi2+HPI)-y1*cos(phi1+HPI)
   yy_=y2*sin(phi2+HPI)-y1*sin(phi1+HPI)
   alpha=atan2(yy_,xx_)

         nrpl1=nrpl1+1
         nrpl2=nrpl2+1

         xx_s=dim1*cos(alpha-alpha1)+y1*cos(phi1+HPI)
         yy_s=dim1*sin(alpha-alpha1)+y1*sin(phi1+HPI)

         call forward(xx_s,yy_s,xx,yy,phi1)

         planetstmp%roots(i,nrpl1)=setangle(atan2(yy-y1,xx))
         planetstmp%nroots(i)=nrpl1

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl2)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl2
         
  ENDIF

  ENDIF

  ENDDO
 ENDDO
ENDIF

!
!        Two spots interception
!
IF (spotstmp%nspots>1) THEN

DO i=1, spotstmp%nspots
 DO j=i+1, spotstmp%nspots
      
         dim1=spotstmp%dim(i)
         dim2=spotstmp%dim(j)
         y1=spotstmp%y(i)
         y2=spotstmp%y(j)
         nrsp1=spotstmp%nroots(i)
         nrsp2=spotstmp%nroots(j)
         phi=spotstmp%phi(j)-spotstmp%phi(i)

         IF ((sin(y1)<epsilon).AND.(sin(y2)<epsilon)) CYCLE    

         cos_rho=cos(y1)*cos(y2)+sin(y1)*sin(y2)*cos(phi)
         sin_rho=sqrt(1.-cos_rho**2)

         IF (sin(y1)<epsilon) THEN
           chi1=PI+phi
           chi2=TWPI
         ELSE IF (sin(y2)<epsilon) THEN
           chi1=0.0D0
           chi2=PI+phi
         ELSE
           chi1=atan2(-sin(phi)*sin(y2)*sin(y1),cos(y2)-cos(y1)*cos_rho)
           chi2=atan2(-sin(phi)*sin(y1)*sin(y2),cos(y1)-cos(y2)*cos_rho)
         ENDIF
         delta1=sqrt((sin_rho*sin(dim2))**2- &
                     (cos(dim1)-cos_rho*cos(dim2))**2)
         delta2=sqrt((sin_rho*sin(dim1))**2- &
                     (cos(dim2)-cos_rho*cos(dim1))**2) 

         if ((delta1.gt.0.0D0).and.(delta2.gt.0.0D0)) then

         gamma1=atan2(delta1,cos(dim2)-cos_rho*cos(dim1))
         gamma2=atan2(delta2,cos(dim1)-cos_rho*cos(dim2))
         cos_rho2=cos_rho*cos(dim1)+sin_rho*sin(dim1)*cos(gamma1)
         sin_rho2=sqrt(1.-cos_rho2**2)
         cos_rho1=cos_rho*cos(dim2)+sin_rho*sin(dim2)*cos(gamma2)
         sin_rho1=sqrt(1.-cos_rho1**2)
         dr1dg2=sin_rho*sin(dim2)*sin(gamma2)/sin_rho1 
         dr2dg1=sin_rho*sin(dim1)*sin(gamma1)/sin_rho2

         call newraps(gamma1,acos(cos_rho2)-dim2,dr2dg1)
         call newraps(gamma2,acos(cos_rho1)-dim1,dr1dg2)

!
!        First root
!
         nrsp1=nrsp1+1
         nrsp2=nrsp2+1
         spotstmp%roots(i,nrsp1)=setangle(chi1-gamma1)
         spotstmp%nroots(i)=nrsp1

         spotstmp%roots(j,nrsp2)=setangle(TWPI-chi2-gamma2)
         spotstmp%nroots(j)=nrsp2


!
!        Second root
!
         nrsp1=nrsp1+1
         nrsp2=nrsp2+1
         spotstmp%roots(i,nrsp1)=setangle(chi1+gamma1)
         spotstmp%nroots(i)=nrsp1

         spotstmp%roots(j,nrsp2)=setangle(TWPI-chi2+gamma2)
         spotstmp%nroots(j)=nrsp2

         endif



  ENDDO
 ENDDO

ENDIF
         
!
!      Interception of planets with spots
!
IF (planetstmp%nplanets>0.AND.spotstmp%nspots>0) THEN
DO i=1, spotstmp%nspots
   DO j=1, planetstmp%nplanets

      dim1=spotstmp%dim(i)
      y1=spotstmp%y(i)
      phi1=spotstmp%phi(i)
      nrsp=spotstmp%nroots(i)
      dim2=planetstmp%dim(j)
      y2=planetstmp%y(j)
      phi2=planetstmp%phi(j)
      nrpl=planetstmp%nroots(j)
      phi=phi2-phi1

      IF (ABS(y2)<epsilon.AND.sin(y1)<epsilon) CYCLE ! Planet and spot centers are coincident along the line of sight

      IF (sin(y1).lt.epsilon) THEN ! Spot center along the line of sight, 
                                   ! this case can be treated 
                                   ! as the interception of two planets

       y1=sin(spotstmp%y(i))
       dim1=sin(spotstmp%dim(i))
       phi1=spotstmp%phi(i)
       nrsp=spotstmp%nroots(i)
       y2=planetstmp%y(j)
       dim2=planetstmp%dim(j)
       phi2=planetstmp%phi(j)
       nrpl=planetstmp%nroots(j)
       phi=phi2-phi1

       IF ((y1/=y2).OR.(dim1/=dim2)) THEN

        dist=sqrt(y1**2+y2**2-2.*y1*y2*cos(phi))
        IF (dist<(dim1+dim2).AND.dist>ABS(dim1-dim2)) THEN
         alpha1=acos((dim1**2+dist**2-dim2**2)/(2.0d0*dim1*dist))
  
         xx_=y2*cos(phi2+HPI)
         yy_=y2*sin(phi2+HPI)
         alpha=setangle(atan2(yy_,xx_))

!
!        First root
!
         nrsp=nrsp+1
         nrpl=nrpl+1

         xx_s=dim1*cos(alpha-alpha1)
         yy_s=dim1*sin(alpha-alpha1)

         call forward(xx_s,yy_s,xx,yy,phi1)

         spotstmp%roots(i,nrsp)=setangle(atan2(yy,xx)+HPI)
         spotstmp%nroots(i)=nrsp

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl

!
!        Second root
!
         nrsp=nrsp+1
         nrpl=nrpl+1

         xx_s=dim1*cos(alpha+alpha1)
         yy_s=dim1*sin(alpha+alpha1)

         call forward(xx_s,yy_s,xx,yy,phi1)

         spotstmp%roots(i,nrsp)=setangle(atan2(yy,xx)+HPI)
         spotstmp%nroots(i)=nrsp

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl

        ELSE IF (dist==(dim1+dim2).OR.(dist==ABS(dim1-dim2).AND.(dim1/=dim2))) THEN

         alpha1=acos((dim1**2+dist**2-dim2**2)/(2.0d0*dim1*dist))
  
         xx_=y2*cos(phi2+HPI)
         yy_=y2*sin(phi2+HPI)
         alpha=atan2(yy_,xx_)

         nrsp=nrsp+1
         nrpl=nrpl+1

         xx_s=dim1*cos(alpha-alpha1)
         yy_s=dim1*sin(alpha-alpha1)

         call forward(xx_s,yy_s,xx,yy,phi1)

         spotstmp%roots(i,nrsp)=setangle(atan2(yy-y1,xx))
         spotstmp%nroots(i)=nrsp

         call forward(xx_s,yy_s,xx,yy,phi2)

         planetstmp%roots(j,nrpl)=setangle(atan2(yy-y2,xx))
         planetstmp%nroots(j)=nrpl
         
        ENDIF
       ENDIF

      ELSE IF (y2<epsilon.AND.dim2<1.0D0) THEN ! Planet center along the line of sight
                                               ! This case can be treated as a special case of
                                               ! the interception of two spots.

         dim1=spotstmp%dim(i)
         dim2=asin(planetstmp%dim(j))
         y1=spotstmp%y(i)
         y2=asin(planetstmp%y(j))
         nrsp=spotstmp%nroots(i)
         nrpl=planetstmp%nroots(j)
         phi=planetstmp%phi(j)-spotstmp%phi(i)

         cos_rho=cos(y1)*cos(y2)+sin(y1)*sin(y2)*cos(phi)
         sin_rho=sqrt(1.-cos_rho**2)

         chi1=0.0D0
         chi2=spotstmp%phi(i)+HPI

         delta1=sqrt((sin_rho*sin(dim2))**2- &
                     (cos(dim1)-cos_rho*cos(dim2))**2)
         delta2=sqrt((sin_rho*sin(dim1))**2- &
                     (cos(dim2)-cos_rho*cos(dim1))**2) 

         IF ((delta1.gt.0.0D0).and.(delta2.gt.0.0D0)) THEN

         gamma1=atan2(delta1,cos(dim2)-cos_rho*cos(dim1))
         gamma2=atan2(delta2,cos(dim1)-cos_rho*cos(dim2))
         cos_rho2=cos_rho*cos(dim1)+sin_rho*sin(dim1)*cos(gamma1)
         sin_rho2=sqrt(1.-cos_rho2**2)
         cos_rho1=cos_rho*cos(dim2)+sin_rho*sin(dim2)*cos(gamma2)
         sin_rho1=sqrt(1.-cos_rho1**2)
         dr1dg2=sin_rho*sin(dim2)*sin(gamma2)/sin_rho1 
         dr2dg1=sin_rho*sin(dim1)*sin(gamma1)/sin_rho2

         call newraps(gamma1,acos(cos_rho2)-dim2,dr2dg1)
         call newraps(gamma2,acos(cos_rho1)-dim1,dr1dg2)

!
!        First root
!
         nrsp=nrsp+1
         nrpl=nrpl+1
         spotstmp%roots(i,nrsp)=setangle(chi1-gamma1)
         spotstmp%nroots(i)=nrsp

         planetstmp%roots(j,nrpl)=setangle(chi2-gamma2)
         planetstmp%nroots(j)=nrpl

!
!        Second root
!
         nrsp=nrsp+1
         nrpl=nrpl+1
         spotstmp%roots(i,nrsp)=setangle(chi1+gamma1)
         spotstmp%nroots(i)=nrsp

         planetstmp%roots(j,nrpl)=setangle(chi2+gamma2)
         planetstmp%nroots(j)=nrpl

         ENDIF
       
      ELSE ! Both the planet and the spot are not along the line of sight

        Nsol=0
        CALL solvequartic(spotstmp,planetstmp,i,j,roots,Nsol)

        IF (Nsol.gt.0) THEN
        DO k=1, Nsol
          nrpl=nrpl+1          
          nrsp=nrsp+1          

          theta_=roots(k)
          x0=y2*cos(HPI+phi)
          y0=y2*sin(HPI+phi)
          do nriter=1, newrapsiter
             rho=sqrt((sin(dim1)*sin(theta_)-x0)**2+ &
                 (sin(y1)*cos(dim1)-cos(y1)* &
                  sin(dim1)*cos(theta_)-y0)**2)
             drho_dalpha=(sin(dim1)*cos(theta_)+ &
                         cos(y1)*sin(dim1)*sin(theta_))/ &
                         rho
             call newraps(theta_,rho-dim2,drho_dalpha)
          enddo

          spotstmp%roots(i,nrsp)=setangle(theta_)
          spotstmp%nroots(i)=nrsp               
        
          xx=sin(dim1)*sin(theta_)
          yy=sin(y1)*cos(dim1)-cos(y1)* &
             sin(dim1)*cos(theta_)

          call forward(xx,yy,xx_,yy_,phi)
            
          planetstmp%roots(j,nrpl)=setangle(atan2(yy_-y2,xx_))
          planetstmp%nroots(j)=nrpl

        ENDDO
        ENDIF

      ENDIF

  ENDDO
ENDDO
ENDIF

!
! Sort the roots
!
IF (planetstmp%nplanets>0) THEN
  DO i=1, planetstmp%nplanets
    IF (planetstmp%nroots(i)>1) THEN
      DO j=1, planetstmp%nroots(i)-1
         DO k=j+1,  planetstmp%nroots(i)
            IF (planetstmp%roots(i,k)<planetstmp%roots(i,j)) THEN
                root_=planetstmp%roots(i,k)
                planetstmp%roots(i,k)=planetstmp%roots(i,j)
                planetstmp%roots(i,j)=root_
            ENDIF
         ENDDO
      ENDDO
    ENDIF
  ENDDO
ENDIF

IF (spotstmp%nspots>0) THEN
  DO i=1, spotstmp%nspots
    IF (spotstmp%nroots(i)>1) THEN
      DO j=1, spotstmp%nroots(i)-1
         DO k=j+1,  spotstmp%nroots(i)
            IF (spotstmp%roots(i,k)<spotstmp%roots(i,j)) THEN
                root_=spotstmp%roots(i,k)
                spotstmp%roots(i,k)=spotstmp%roots(i,j)
                spotstmp%roots(i,j)=root_
            ENDIF
         ENDDO
      ENDDO
    ENDIF
  ENDDO
ENDIF

IF (startmp%nroots>1) THEN
   DO j=1, startmp%nroots-1
      DO k=j+1,  startmp%nroots
         IF (startmp%roots(k)<startmp%roots(j)) THEN
                root_=startmp%roots(k)
                startmp%roots(k)=startmp%roots(j)
                startmp%roots(j)=root_
         ENDIF
      ENDDO
   ENDDO
ENDIF

!
! Checks if all roots are correctly set
!
IF (startmp%nroots>0) THEN 
  DO i=1, startmp%nroots
     IF (startmp%roots(i)/=startmp%roots(i)) THEN
        status=38
     ENDIF
  ENDDO
ENDIF

IF (spotstmp%nspots>0) THEN 
  DO i=1, spotstmp%nspots
   DO k=1, spotstmp%nroots(i)
     IF (spotstmp%roots(i,k)/=spotstmp%roots(i,k)) THEN
        status=38
     ENDIF
    ENDDO
  ENDDO
ENDIF

IF (planetstmp%nplanets>0) THEN 
  DO i=1, planetstmp%nplanets
   DO k=1, planetstmp%nroots(i)
     IF (planetstmp%roots(i,k)/=planetstmp%roots(i,k)) THEN
        status=38
     ENDIF
   ENDDO
  ENDDO
ENDIF

END SUBROUTINE INTERSECT

SUBROUTINE CLASSIFYARCS(startmp,planetstmp,spotstmp)
IMPLICIT NONE

INTEGER           :: i, j, k
TYPE(startmp_)    :: startmp
TYPE(planetstmp_) :: planetstmp
TYPE(spotstmp_)   :: spotstmp
REAL(DP)          :: lastffact, met, zmid, ffact
REAL(DP)          :: curv1, curv2, dotpcurv

CALL SET_MIDANGLE(startmp,planetstmp,spotstmp)

!
!             Conditions
!

!
!      Arc on the stellar border
!

DO i=1, startmp%nroots

   IF (planetstmp%nplanets>0) THEN
     DO j=1, planetstmp%nplanets

            IF (ABS(planetstmp%dim(j)-1.0D0)<epsilon.AND.planetstmp%y(j)<epsilon) THEN ! Border coincident with planet
                startmp%fgarc(i)=1
                startmp%arcffact(i)=1.0D0
                GO TO 55            
            ENDIF
            IF (planetstmp%dim(j)>=(planetstmp%y(j)+1.0D0)) THEN ! Border totally occulted by planet
                startmp%fgarc(i)=1
                startmp%arcffact(i)=1.0D0
                GO TO 55            
            ENDIF
            CALL GETMET(startmp,planetstmp,i,j,met)
            IF (met<=0.) THEN                                    ! Arc inside planet
                startmp%fgarc(i)=1
                startmp%arcffact(i)=1.0D0
                GO TO 55
            ENDIF    
     ENDDO
   ENDIF

   IF (spotstmp%nspots>0) THEN
     lastffact=0.0D0
     startmp%sumffact(i)=0.0D0
     startmp%arcffact(i)=0.0D0
     DO j=1, spotstmp%nspots           
            IF ((ABS(spotstmp%dim(i)-HPI)<epsilon).AND.(ABS(spotstmp%y(i)-PI)<epsilon)) CYCLE ! Half hemispheric spot 
                                                                                              ! totally in the invisible hemisphere
            CALL GETMET(startmp,spotstmp,i,j,met)
            IF ((met<=0.).OR.((ABS(spotstmp%dim(i)-HPI)<epsilon).AND.(ABS(spotstmp%y(i))<epsilon))) THEN 
                                                                          ! Border inside a spot or 
                                                                          ! coincident with a half-emispheric spot totally visible
               IF (spotstmp%ffact(j)/=lastffact) THEN
                   startmp%fgarc(i)=1
                   startmp%sumffact(i)=startmp%sumffact(i)+spotstmp%ffact(j)
                   IF (startmp%sumffact(i)>1.0D0) CALL errmsg(26)         ! Overlapping regions with contrast ratio > 1
                   lastffact=spotstmp%ffact(j)
               ENDIF
            ENDIF    
     ENDDO
     startmp%arcffact(i)=startmp%sumffact(i)
   ENDIF

55 CONTINUE
ENDDO

!
!      Arc on spots
!
IF (spotstmp%nspots>0) THEN

DO i=1, spotstmp%nspots

DO k=1, spotstmp%nroots(i)

   spotstmp%arcffact(i,k)=0.0D0
   spotstmp%sumffact(i,k)=0.0D0
   lastffact=0.
   CALL GETZ(spotstmp,i,k,zmid)
   IF (zmid<0.) GO TO 65                                                     ! Arc in the invisible hemisphere
   IF ((ABS(spotstmp%dim(i)-HPI)<epsilon).AND.(ABS(spotstmp%y(i)-PI)<epsilon.OR.ABS(spotstmp%y(i))<epsilon)) GO TO 65 
                                                                             ! Half hemispheric spot totally in the visible/invisible hemisphere
                                                                             ! These cases have been already accounted for the stellar border

   IF (planetstmp%nplanets>0) THEN
     DO j=1, planetstmp%nplanets
            CALL GETMET(spotstmp,planetstmp,i,k,j,met)

            IF (ABS(met)<epsilon) THEN
                 IF (spotstmp%y(i)==0.0D0) THEN         ! Spot totally occulted by the planet and coincident with it
                      spotstmp%fgarc(i,k)=0
                      spotstmp%arcffact(i,k)=0.0D0
                     GO TO 65
                 ELSE IF (spotstmp%y(i)==PI) THEN       ! Spot complementary to the planet profile
                      spotstmp%fgarc(i,k)=1
                      spotstmp%arcffact(i,k)=spotstmp%ffact(i) 
                 ENDIF
            ELSE IF (met<0.0D0) THEN                    ! Arc inside a planet in the visible hemisphere
                spotstmp%fgarc(i,k)=0
                spotstmp%arcffact(i,k)=0.0D0
                GO TO 65
            ELSE IF (met>0.0D0) THEN                    ! Arc outside a planet in the visible hemisphere
                spotstmp%fgarc(i,k)=1
                spotstmp%arcffact(i,k)=spotstmp%ffact(i)
            ENDIF    
     ENDDO
   ELSE
                spotstmp%fgarc(i,k)=1
                spotstmp%arcffact(i,k)=spotstmp%ffact(i) 
   ENDIF

   IF (spotstmp%nspots>1) THEN
     ffact=spotstmp%ffact(i)
     lastffact=spotstmp%ffact(i)
     DO j=1, spotstmp%nspots           
        IF (j/=i) THEN
            CALL GETMET(spotstmp,i,k,j,met)
            IF (met<0.) THEN ! Arc inside a spot in the visible hemisphere
              IF (spotstmp%ffact(j)/=spotstmp%ffact(i)) THEN ! With unequal contrast ratios
                   spotstmp%fgarc(i,k)=1
                   IF (spotstmp%ffact(i)/=lastffact) THEN
                    ffact=ffact+spotstmp%ffact(j)
                    IF (ffact>1.0D0) CALL errmsg(26)  ! Overlapping regions with contrast ratio > 1
                    lastffact=spotstmp%ffact(j)
                   ENDIF
              ELSE IF (spotstmp%ffact(j)==spotstmp%ffact(i)) THEN ! With equal contrast ratios
                   spotstmp%fgarc(i,k)=0
                   EXIT
              ENDIF    
            ELSE             ! Arc outside a spot or on a complementary spot, 
                             ! note that coincident spots have been already considered
                   spotstmp%fgarc(i,k)=1
            ENDIF
        ENDIF
     ENDDO
     IF (spotstmp%fgarc(i,k)==1) THEN
         spotstmp%arcffact(i,k)=spotstmp%ffact(i)
     ENDIF
   ENDIF

65 CONTINUE
ENDDO

ENDDO

ENDIF

!
!      Arc on planets.
!
IF (planetstmp%nplanets>0) THEN

DO i=1, planetstmp%nplanets

DO k=1, planetstmp%nroots(i)
   planetstmp%arcffact(i,k)=1.0D0
   lastffact=0.

   CALL GETZ(planetstmp,i,k,zmid)
   IF (zmid==0.0D0) GO TO 75               ! Arc outside the star or coincident with the stellar border

   IF (planetstmp%nplanets>1) THEN
     DO j=1, planetstmp%nplanets
        IF (j/=i) THEN
            CALL GETMET(planetstmp,i,k,j,met)
            IF (met>=0.0D0) THEN           ! Arc external to a planet
                                           ! Note that coincident planet profiles have been already considered
                planetstmp%fgarc(i,k)=1
            ELSE
                planetstmp%fgarc(i,k)=0    ! Arc internal to a planet
                GO TO 75 
            ENDIF
         ENDIF
     ENDDO
   ELSE
                planetstmp%fgarc(i,k)=1    ! Arc internal to the star with no other planets
   ENDIF  

    IF (spotstmp%nspots>0) THEN
      planetstmp%sumffact(i,k)=0.0D0
      DO j=1, spotstmp%nspots           

            CALL GETMET(planetstmp,spotstmp,i,k,j,met,'PS')
            IF (met<0.0D0) THEN            ! Arc inside a spot
              IF (spotstmp%ffact(j)/=lastffact) THEN
                  planetstmp%fgarc(i,k)=1
                  planetstmp%sumffact(i,k)=planetstmp%sumffact(i,k)+spotstmp%ffact(j)
                  IF (planetstmp%sumffact(i,k)>1.0D0) CALL errmsg(26) ! Overlapping regions with contrast ratio > 1
                  lastffact=spotstmp%ffact(j)
              ENDIF
            ELSE IF (met>=0.0D0) THEN      ! Arc on a coincident or complementary spot to the planet profile or external to a spot
                  planetstmp%fgarc(i,k)=1
            ENDIF
      ENDDO
      planetstmp%arcffact(i,k)=planetstmp%arcffact(i,k)-planetstmp%sumffact(i,k)
    ENDIF

75 CONTINUE
ENDDO

ENDDO

ENDIF
END SUBROUTINE CLASSIFYARCS

SUBROUTINE SET_MIDANGLE(startmp,planetstmp,spotstmp)
IMPLICIT NONE

INTEGER :: i, j, status
TYPE(startmp_)    :: startmp
TYPE(planetstmp_) :: planetstmp
TYPE(spotstmp_)   :: spotstmp

IF (startmp%nroots==0) THEN
  IF (ASSOCIATED(startmp%midangle)) DEALLOCATE(startmp%midangle)
  IF (ASSOCIATED(startmp%arcffact)) DEALLOCATE(startmp%arcffact)
  ALLOCATE(startmp%midangle(1), STAT=status)
  IF (status/=0) call errmsg(31)
  ALLOCATE(startmp%arcffact(1), STAT=status)
  IF (status/=0) call errmsg(31)
  startmp%roots(1)=0.0D0
  startmp%midangle(1)=PI
  startmp%sumffact(1)=0.0D0
  startmp%nroots=1
  startmp%arcffact(1)=0.0D0
ELSE IF (startmp%nroots==1) THEN
  IF (ASSOCIATED(startmp%midangle)) DEALLOCATE(startmp%midangle)
  IF (ASSOCIATED(startmp%arcffact)) DEALLOCATE(startmp%arcffact)
  ALLOCATE(startmp%midangle(1), STAT=status)
  IF (status/=0) call errmsg(31)
  ALLOCATE(startmp%arcffact(1), STAT=status)
  IF (status/=0) call errmsg(31)
  startmp%midangle(1)=setangle(startmp%roots(1)+PI)
  startmp%sumffact(1)=0.0D0
  startmp%arcffact(1)=0.0D0
ELSE IF (startmp%nroots>1) THEN
  IF (ASSOCIATED(startmp%midangle)) DEALLOCATE(startmp%midangle)
  IF (ASSOCIATED(startmp%arcffact)) DEALLOCATE(startmp%arcffact)
  ALLOCATE(startmp%midangle(startmp%nroots), STAT=status)
  IF (status/=0) call errmsg(31)
  ALLOCATE(startmp%arcffact(startmp%nroots), STAT=status)
  IF (status/=0) call errmsg(31)
  DO i=1, startmp%nroots-1
      startmp%midangle(i)=setangle(0.5D0*(startmp%roots(i)+startmp%roots(i+1)))
      startmp%sumffact(i)=0.0D0
      startmp%arcffact(i)=0.0D0
  ENDDO
  startmp%midangle(startmp%nroots)=setangle(0.5D0*(startmp%roots(startmp%nroots)+TWPI+startmp%roots(1)))
  startmp%sumffact(startmp%nroots)=0.0D0
  startmp%arcffact(startmp%nroots)=0.0D0
  CALL CHECK_TANGENT_POINT_STAR(startmp,spotstmp,planetstmp)
ENDIF

IF (planetstmp%nplanets>0) THEN
  DO i=1, planetstmp%nplanets
   IF (planetstmp%nroots(i)==0) THEN
    planetstmp%midangle(i,1)=PI
    planetstmp%nroots(i)=1
    planetstmp%sumffact(i,1)=0.0D0
    planetstmp%arcffact(i,1)=0.0D0
   ELSE IF (planetstmp%nroots(i)==1) THEN
    planetstmp%midangle(i,1)=setangle(planetstmp%roots(i,1)+PI)
    planetstmp%sumffact(i,1)=0.0D0
    planetstmp%arcffact(i,1)=0.0D0
   ELSE
    DO j=1, planetstmp%nroots(i)-1
      planetstmp%midangle(i,j)=setangle(0.5D0*(planetstmp%roots(i,j)+planetstmp%roots(i,j+1)))
      planetstmp%sumffact(i,j)=0.0D0
      planetstmp%arcffact(i,j)=0.0D0
    ENDDO
    planetstmp%midangle(i,planetstmp%nroots(i))=setangle(0.5D0*(planetstmp%roots(i,planetstmp%nroots(i))+ & 
                                                TWPI+planetstmp%roots(i,1)))
    planetstmp%sumffact(i,planetstmp%nroots(i))=0.0D0
    planetstmp%arcffact(i,planetstmp%nroots(i))=0.0D0
   ENDIF
  ENDDO
  CALL CHECK_TANGENT_POINT_PLANETS(startmp,spotstmp,planetstmp)
ENDIF

IF (spotstmp%nspots>0) THEN
  DO i=1, spotstmp%nspots
   IF (spotstmp%nroots(i)==0) THEN
    spotstmp%midangle(i,1)=PI
    spotstmp%nroots(i)=1
    spotstmp%sumffact(i,1)=0.0D0
    spotstmp%arcffact(i,1)=0.0D0
   ELSE IF (spotstmp%nroots(i)==1) THEN
    spotstmp%midangle(i,1)=setangle(spotstmp%roots(i,1)+PI)
    spotstmp%sumffact(i,1)=0.0D0
    spotstmp%arcffact(i,1)=0.0D0
   ELSE
    DO j=1, spotstmp%nroots(i)-1
      spotstmp%midangle(i,j)=setangle(0.5D0*(spotstmp%roots(i,j)+spotstmp%roots(i,j+1)))
      spotstmp%sumffact(i,j)=0.0D0
      spotstmp%arcffact(i,j)=0.0D0
    ENDDO
    spotstmp%midangle(i,spotstmp%nroots(i))=setangle(0.5D0*(spotstmp%roots(i,spotstmp%nroots(i))+TWPI+spotstmp%roots(i,1)))
    spotstmp%sumffact(i,spotstmp%nroots(i))=0.0D0
    spotstmp%arcffact(i,spotstmp%nroots(i))=0.0D0
   ENDIF
  ENDDO
  CALL CHECK_TANGENT_POINT_SPOTS(startmp,spotstmp,planetstmp)
ENDIF

END SUBROUTINE SET_MIDANGLE

SUBROUTINE CHECK_TANGENT_POINT_STAR(startmp,spotstmp,planetstmp)
IMPLICIT NONE

INTEGER            :: i,j,tg_point,iter, status
INTEGER, PARAMETER :: niter=3
REAL(DP), PARAMETER    :: EPS=0.01D0, PERC=0.9D0
TYPE(startmp_)     :: startmp
TYPE(planetstmp_)  :: planetstmp
TYPE(spotstmp_)    :: spotstmp
REAL(DP)               :: minmet,met

IF (ASSOCIATED(startmp%tgpoint)) DEALLOCATE(startmp%tgpoint)
IF (startmp%nroots==0.OR.startmp%nroots==1) THEN
    ALLOCATE(startmp%tgpoint(1), STAT=status)
    IF (status/=0) call errmsg(32)
    startmp%tgpoint=0
ELSE
    ALLOCATE(startmp%tgpoint(startmp%nroots), STAT=status)
    IF (status/=0) call errmsg(32)
    startmp%tgpoint=0
ENDIF

IF (startmp%nroots>0) THEN
DO i=1, startmp%nroots

   tg_point=1
   iter=0
   DO WHILE (tg_point==1.AND.iter<niter)

   minmet=1.0E+10
   IF (planetstmp%nplanets>0) THEN
    DO j=1, planetstmp%nplanets
         CALL GETMET(startmp,planetstmp,i,j,met)       
         IF (ABS(met)<minmet) minmet=met
    ENDDO
   ENDIF

   IF (spotstmp%nspots>0) THEN
    DO j=1, spotstmp%nspots
         CALL GETMET(startmp,spotstmp,i,j,met)       
         IF (ABS(met)<minmet) minmet=met
    ENDDO
   ENDIF

   IF (ABS(minmet)<EPS) THEN
      
        IF (startmp%roots(i)<startmp%midangle(i)) THEN
            startmp%midangle(i)=startmp%roots(i)+ &
                     PERC*(startmp%midangle(i)-startmp%roots(i))
        ELSE
            startmp%midangle(i)=setangle(startmp%roots(i)+ &
                     PERC*(TWPI-startmp%roots(i)+startmp%midangle(i)))
        ENDIF

      tg_point=1
   ELSE
      tg_point=0
   ENDIF

      iter=iter+1
   ENDDO

   startmp%tgpoint(i)=tg_point
ENDDO
ENDIF
END SUBROUTINE CHECK_TANGENT_POINT_STAR

SUBROUTINE CHECK_TANGENT_POINT_SPOTS(startmp,spotstmp,planetstmp)
IMPLICIT NONE

INTEGER            :: i,j,k,iter,tg_point, status
INTEGER, PARAMETER :: niter=3
REAL(DP), PARAMETER    :: EPS=0.01D0, PERC=0.9D0
TYPE(startmp_)     :: startmp
TYPE(planetstmp_)  :: planetstmp
TYPE(spotstmp_)    :: spotstmp
REAL(DP)               :: minmet,met

IF (ASSOCIATED(spotstmp%tgpoint)) DEALLOCATE(spotstmp%tgpoint)
ALLOCATE(spotstmp%tgpoint(spotstmp%nspots,planetstmp%nplanets*4+(spotstmp%nspots-1)*2+2), STAT=status)
spotstmp%tgpoint=0
IF (status/=0) call errmsg(33)

DO i=1, spotstmp%nspots
   IF (spotstmp%nroots(i)>0) THEN
   DO k=1, spotstmp%nroots(i)

   tg_point=1
   iter=0
   DO WHILE (tg_point==1.AND.iter<niter)

    minmet=1.0E+10
    CALL GETMET(startmp,spotstmp,i,k,met)       
    IF (ABS(met)<minmet) minmet=met

    IF (planetstmp%nplanets>0) THEN
     DO j=1, planetstmp%nplanets
         CALL GETMET(spotstmp,planetstmp,i,k,j,met)       
         IF (ABS(met)<minmet) minmet=met
     ENDDO
    ENDIF

    IF (spotstmp%nspots>1) THEN
     DO j=1, spotstmp%nspots
        IF (i/=j) THEN
          CALL GETMET(spotstmp,i,k,j,met)       
          IF (ABS(met)<minmet) minmet=met
        ENDIF
     ENDDO
    ENDIF

    IF (ABS(minmet)<EPS) THEN
        IF (spotstmp%roots(i,k)<spotstmp%midangle(i,k)) THEN
            spotstmp%midangle(i,k)=spotstmp%roots(i,k)+ &
                     PERC*(spotstmp%midangle(i,k)-spotstmp%roots(i,k))
        ELSE
            spotstmp%midangle(i,k)=setangle(spotstmp%roots(i,k)+ &
                     PERC*(TWPI-spotstmp%roots(i,k)+spotstmp%midangle(i,k)))
        ENDIF
        tg_point=1
    ELSE
        tg_point=0
    ENDIF

   iter=iter+1
   ENDDO
   spotstmp%tgpoint(i,k)=tg_point

   ENDDO
   ENDIF
ENDDO

END SUBROUTINE CHECK_TANGENT_POINT_SPOTS

SUBROUTINE CHECK_TANGENT_POINT_PLANETS(startmp,spotstmp,planetstmp)
IMPLICIT NONE

INTEGER            :: i, j, k, iter, tg_point, status
INTEGER, PARAMETER :: niter=3
REAL(DP), PARAMETER    :: EPS=0.01D0, PERC=0.9D0
TYPE(startmp_)     :: startmp
TYPE(planetstmp_)  :: planetstmp
TYPE(spotstmp_)    :: spotstmp
REAL(DP)           :: minmet,met
CHARACTER          :: flag

IF (ASSOCIATED(planetstmp%tgpoint)) DEALLOCATE(planetstmp%tgpoint)
ALLOCATE(planetstmp%tgpoint(planetstmp%nplanets,(planetstmp%nplanets-1)*2+spotstmp%nspots*4+2), STAT=status)
IF (status/=0) call errmsg(34)
planetstmp%tgpoint=0

DO i=1, planetstmp%nplanets
   IF (planetstmp%nroots(i)>0) THEN
   DO k=1, planetstmp%nroots(i)

   tg_point=1
   iter=0
   DO WHILE (tg_point==1.AND.iter<niter)

    minmet=1.0E+10

    CALL GETMET(startmp,planetstmp,i,k,met)       
    IF (ABS(met)<minmet) minmet=met

    IF (planetstmp%nplanets>1) THEN
     DO j=1, planetstmp%nplanets
        IF (i/=j) THEN
         CALL GETMET(planetstmp,i,k,j,met)       
         IF (ABS(met)<minmet) minmet=met
        ENDIF
     ENDDO
    ENDIF

    IF (spotstmp%nspots>0) THEN
     DO j=1, spotstmp%nspots
          CALL GETMET(planetstmp,spotstmp,i,k,j,met,flag)       
          IF (ABS(met)<minmet) minmet=met
     ENDDO
    ENDIF

    IF (ABS(minmet)<EPS) THEN
        IF (planetstmp%roots(i,k)<planetstmp%midangle(i,k)) THEN
            planetstmp%midangle(i,k)=planetstmp%roots(i,k)+ &
                     PERC*(planetstmp%midangle(i,k)-planetstmp%roots(i,k))
        ELSE
            planetstmp%midangle(i,k)=setangle(planetstmp%roots(i,k)+ &
                     PERC*(TWPI-planetstmp%roots(i,k)+planetstmp%midangle(i,k)))
        ENDIF

        tg_point=1
    ELSE
        tg_point=0
    ENDIF

        iter=iter+1
   ENDDO

   planetstmp%tgpoint(i,k)=tg_point

   ENDDO
   ENDIF
ENDDO

END SUBROUTINE CHECK_TANGENT_POINT_PLANETS

SUBROUTINE GETZ_SPOTS(spotstmp,i,k,zz)
IMPLICIT NONE

INTEGER           :: i, k
TYPE(spotstmp_)   :: spotstmp
REAL(DP)          :: zz

zz=cos(spotstmp%y(i))*cos(spotstmp%dim(i))+sin(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(spotstmp%midangle(i,k))
END SUBROUTINE GETZ_SPOTS

SUBROUTINE GETZ_PLANETS(planetstmp,i,k,zz)
IMPLICIT NONE

INTEGER           :: i, k
TYPE(planetstmp_) :: planetstmp
REAL(DP)          :: xx,yy,zz

xx=planetstmp%dim(i)*cos(planetstmp%midangle(i,k))
yy=planetstmp%dim(i)*sin(planetstmp%midangle(i,k))+planetstmp%y(i)

IF ((xx**2+yy**2)>=1.0D0) THEN
     zz=0.0D0
ELSE
     zz=sqrt(1.-xx**2-yy**2)
ENDIF   
END SUBROUTINE GETZ_PLANETS

SUBROUTINE GETMET_BORDER_PLANETS(startmp,planetstmp,i,j,met)
IMPLICIT NONE

INTEGER :: i, j
REAL(DP)    :: met
TYPE(startmp_)    :: startmp
TYPE(planetstmp_) :: planetstmp
REAL(DP) :: xx, yy, phi, xx_, yy_

xx=cos(startmp%midangle(i))
yy=sin(startmp%midangle(i))
phi=planetstmp%phi(j)
call FORWARD(xx,yy,xx_,yy_,phi)

met=(sqrt((xx_)**2+(yy_-planetstmp%y(j))**2)-planetstmp%dim(j))/planetstmp%dim(j)

END SUBROUTINE GETMET_BORDER_PLANETS

SUBROUTINE GETMET_BORDER_SPOTS(startmp,spotstmp,i,j,met)
IMPLICIT NONE

INTEGER :: i, j
REAL(DP)    :: met
TYPE(startmp_)    :: startmp
TYPE(spotstmp_) :: spotstmp
REAL(DP) :: xx, yy, phi, xx_, yy_, yc_, zc_

xx=cos(startmp%midangle(i))
yy=sin(startmp%midangle(i))

phi=spotstmp%phi(j)

CALL FORWARD(xx,yy,xx_,yy_,phi)
yc_=sin(spotstmp%y(j))

met=(acos(yc_*yy_)-spotstmp%dim(j))/spotstmp%dim(j)

END SUBROUTINE GETMET_BORDER_SPOTS

SUBROUTINE GETMET_PLANETS_SPOTS(planetstmp,spotstmp,i,k,j,met,flag)
IMPLICIT NONE

INTEGER :: i, j, k
REAL(DP)    :: met
CHARACTER :: flag
TYPE(planetstmp_) :: planetstmp
TYPE(spotstmp_)    :: spotstmp
REAL(DP) :: xx, yy, phi, xx_, yy_, zz_, xc_, yc_, zc_

xx=planetstmp%dim(i)*cos(planetstmp%midangle(i,k))
yy=planetstmp%dim(i)*sin(planetstmp%midangle(i,k))+planetstmp%y(i)
phi=spotstmp%phi(j)-planetstmp%phi(i)

CALL FORWARD(xx,yy,xx_,yy_,phi)

IF ((xx_**2+yy_**2)>=1.0D0) THEN
     zz_=0.0D0
ELSE
     zz_=sqrt(1.0D0-xx_**2-yy_**2)
ENDIF   

yc_=sin(spotstmp%y(j))
zc_=cos(spotstmp%y(j))
met=(acos(yc_*yy_+zc_*zz_)-spotstmp%dim(j))/spotstmp%dim(j)

END SUBROUTINE GETMET_PLANETS_SPOTS

SUBROUTINE GETMET_SPOTS_PLANETS(spotstmp,planetstmp,i,k,j,met)
IMPLICIT NONE

INTEGER :: i, j, k
REAL(DP)    :: met
TYPE(spotstmp_)    :: spotstmp
TYPE(planetstmp_)    :: planetstmp
REAL(DP) :: xx, yy, phi, xx_, yy_

xx=sin(spotstmp%dim(i))*sin(spotstmp%midangle(i,k))
yy=sin(spotstmp%y(i))*cos(spotstmp%dim(i))-cos(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(spotstmp%midangle(i,k))
phi=planetstmp%phi(j)-spotstmp%phi(i)

call FORWARD(xx,yy,xx_,yy_,phi)

met=(sqrt((xx_)**2+(yy_-planetstmp%y(j))**2)-planetstmp%dim(j))/planetstmp%dim(j)

END SUBROUTINE GETMET_SPOTS_PLANETS

SUBROUTINE GETMET_SPOTS_SPOTS(spotstmp,i,k,j,met)
IMPLICIT NONE

INTEGER :: i, j, k
REAL(DP)    :: met
TYPE(spotstmp_)    :: spotstmp
REAL(DP) :: xx, yy, phi, xx_, yy_, zz_, xc_, yc_, zc_

xx=sin(spotstmp%dim(i))*sin(spotstmp%midangle(i,k))
yy=sin(spotstmp%y(i))*cos(spotstmp%dim(i))-cos(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(spotstmp%midangle(i,k))
phi=spotstmp%phi(j)-spotstmp%phi(i)

call FORWARD(xx,yy,xx_,yy_,phi)

zz_=cos(spotstmp%y(i))*cos(spotstmp%dim(i))+sin(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(spotstmp%midangle(i,k))

yc_=sin(spotstmp%y(j))
zc_=cos(spotstmp%y(j))
met=(acos(yc_*yy_+zc_*zz_)-spotstmp%dim(j))/spotstmp%dim(j)

END SUBROUTINE GETMET_SPOTS_SPOTS

SUBROUTINE GETMET_PLANETS_PLANETS(planetstmp,i,k,j,met)
IMPLICIT NONE

INTEGER :: i, j, k
REAL(DP)    :: met
TYPE(planetstmp_)    :: planetstmp
REAL(DP) :: xx, yy,phi, xx_, yy_

xx=planetstmp%dim(i)*cos(planetstmp%midangle(i,k))
yy=planetstmp%dim(i)*sin(planetstmp%midangle(i,k))+planetstmp%y(i)
phi=planetstmp%phi(j)-planetstmp%phi(i)

call FORWARD(xx,yy,xx_,yy_,phi)

met=(sqrt((xx_)**2+(yy_-planetstmp%y(j))**2)-planetstmp%dim(j))/planetstmp%dim(j)

END SUBROUTINE GETMET_PLANETS_PLANETS

SUBROUTINE FORWARD(x,y,xx,yy,phi)
IMPLICIT NONE

REAL(DP) :: x,y,xx,yy,phi

!
!     Transform coordinates from the reference system of
!     the first object to the rotated system of the second.  
!

      xx=x*cos(phi)+y*sin(phi)
      yy=-x*sin(phi)+y*cos(phi)

END SUBROUTINE FORWARD

SUBROUTINE BACKWARD(x,y,xx,yy,phi)
IMPLICIT NONE

REAL(DP) :: x,y,xx,yy,phi

!
!     Transform coordinates from the rotated 
!     system of the second object to the reference system of
!     the first.
!

      xx=x*cos(phi)-y*sin(phi)
      yy=x*sin(phi)+y*cos(phi)

END SUBROUTINE BACKWARD

SUBROUTINE SOLVEQUARTIC(spotstmp,planetstmp,i,j,roots,Nsol)
IMPLICIT NONE

INTEGER :: Nsol
INTEGER :: i,j,ii,jj
REAL(dp) :: A,B,C,D,E
REAL(dp) :: delta_phi,theta_0,sintheta_p,alpha,r_p
REAL(dp) :: sol(4),soli(4),roots(4),delta
REAL(dp), DIMENSION(0:4) :: coeff 
REAL(dp) :: x,y
REAL(dp) :: p,q,v2
REAL(dp) :: sin_alpha,cos_alpha,cos_phi_phi0,sin_phi_phi0
REAL(dp) :: cos_phi_phip,sin_phi_phip
TYPE(spotstmp_)   :: spotstmp
TYPE(planetstmp_) :: planetstmp
REAL(DP) :: dim1,dim2,y1,y2,phi1,phi2
      
      dim1=spotstmp%dim(i)
      y1=spotstmp%y(i)
      phi1=spotstmp%phi(i)
      dim2=planetstmp%dim(j)
      y2=planetstmp%y(j)
      phi2=planetstmp%phi(j)

      delta_phi=dble(phi2-phi1)
      alpha=dble(dim1)
      theta_0=dble(y1)
      r_p=dble(dim2)
      sintheta_p=dble(y2)

      IF (ABS(delta_phi)<epsilon) delta_phi=epsilon
      IF (ABS(delta_phi-PI)<epsilon.AND.delta_phi<=PI) delta_phi=PI-epsilon
      IF (ABS(delta_phi-PI)<epsilon.AND.delta_phi>=PI) delta_phi=PI+epsilon

      p = sin(theta_0) / sintheta_p * sin(delta_phi)

      q = sin(theta_0) / sintheta_p * cos(delta_phi)

      v2= sintheta_p**2 - r_p**2

      A = -0.25d0 * (p**2 + q**2)

      B = q * cos(theta_0)

      C = 0.5d0 * (v2 + 1.0d0) * (p**2 + q**2) - &
          ((cos(theta_0))**2 + q * cos(alpha)) - &
          (p**2) * sintheta_p**2

      D = 2.0d0 * cos(theta_0) * cos(alpha) - q * cos(theta_0) * &
          (v2 + 1.0d0)

      E = -0.25d0 * (p**2) * v2 * (v2 + 2.0d0) - 0.25d0 * (q**2) * &
          (v2 + 1.0d0)**2 - (cos(alpha))**2 + q * cos(alpha) * &
          (v2 + 1.0d0) - 0.25d0 * (p**2) + (p**2) * sintheta_p**2

      coeff(4)=A 
      coeff(3)=B 
      coeff(2)=C 
      coeff(1)=D 
      coeff(0)=E 
      Nsol=0
      call QUARTIC(coeff,sol,soli,Nsol)

      if (Nsol.gt.0) then
        jj=0
        do ii=1, Nsol

           if(soli(ii)==0.0D0.AND.abs(sol(ii)).le.1.0d0) then         

             cos_alpha=(sol(ii)-cos(alpha)*cos(theta_0))/ &
                        (sin(alpha)*sin(theta_0))

             cos_phi_phip=(v2+1.0d0-sol(ii)**2)/ &
                (2.0d0*sintheta_p*sqrt(1.0d0-sol(ii)**2))

             cos_phi_phi0=(cos(alpha)-cos(theta_0)*sol(ii))/ &
                (sin(theta_0)*sqrt(1.0d0-sol(ii)**2))

             jj=jj+1

             sin_phi_phip=(cos_phi_phip*cos(delta_phi)-cos_phi_phi0)/ &
                sin(delta_phi)

             sin_phi_phi0=sin_phi_phip*cos(delta_phi)+ &
                cos_phi_phip*sin(delta_phi)

             sin_alpha=-sin_phi_phi0 / sin(alpha)*sqrt(1.0d0-sol(ii)**2)

             roots(jj)=atan2(sin_alpha,cos_alpha)

           endif
        enddo
        Nsol=jj
      endif

END SUBROUTINE SOLVEQUARTIC

!+
SUBROUTINE QUARTIC(a,sol,soli,Nsol)
! ---------------------------------------------------------------------------
! PURPOSE - Test the functioning of the procedures in the module
!    PolynomialRoots and compare to those in the original module
!    by Alan Miller
! AUTHORS - Alan Miller,  CSIRO Mathematical & Information Sciences
!                         CLAYTON, VICTORIA, AUSTRALIA 3169
!                         http://www.mel.dms.csiro.au/~alan
!           Ralph L. Carmichael, Public Domain Aeronautical Software
!                         http://www.pdas.com
!     REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
! 27Feb97  1.0   AM    Original coding
! 16Jul98  1.1   RLC   Numerous style changes
!
! Slightly modified by MM.

  USE PolynomialRoots
  IMPLICIT NONE

  INTEGER,PARAMETER:: DP=KIND(1.0D0)
!  CHARACTER(LEN=*),PARAMETER:: FMT = "(2ES20.12)"

  CHARACTER(LEN=80):: buffer
  INTEGER:: i, Nsol
  REAL(dp):: sol(4), soli(4)
  REAL(dp),DIMENSION(0:4):: a
  COMPLEX(dp),DIMENSION(4):: z
!----------------------------------------------------------------------------
!  WRITE(*,*) "Solve quadratic, cubic or quartic eqns. with real coefficients"
!  WRITE(*,*) " "
!  WRITE(*,*) "  A*x**4 + B*x**3 + C*x**2 + D*x + E = 0"
!  WRITE(*,*) " "

!  WRITE(*,*) "Enter the coefficient of the constant term:"
!  READ(*,'(A)') buffer
!  READ(buffer,'(F80.0)') a(0)

!  WRITE(*,*) "Enter the coefficient of the linear term:"
!  READ(*,'(A)') buffer
!  READ(buffer,'(F80.0)') a(1)

!  WRITE(*,*) "Enter the coefficient of the quadratic term:"
!  READ(*,'(A)') buffer
!  READ(buffer,'(F80.0)') a(2)

!  WRITE(*,*) "Enter the coefficient of the cubic term:"
!  READ(*,'(A)') buffer
!  READ(buffer,'(F80.0)') a(3)

!  WRITE(*,*) "Enter the coefficient of the quartic term:"
!  READ(*,'(A)') buffer
!  READ(buffer,'(F80.0)') a(4)

  DO i=1, 4
     sol(i)=0.
     soli(i)=0.
  ENDDO

  IF (a(4) /= 0.0) THEN

    CALL QuarticRoots(a, z)
!    WRITE(*,*) " Roots: REAL PART   IMAGINARY PART"
!    WRITE(*,FMT) (DBLE(z(i)), AIMAG(z(i)), i=1,4)
    Nsol=4
    DO i=1, Nsol
       sol(i)=DBLE(z(i))
       soli(i)=AIMAG(z(i))
    ENDDO
  ELSE IF(a(3) /= 0.0) THEN

    CALL CubicRoots(a, z)
!    WRITE(*,*) " Roots: REAL PART   IMAGINARY PART"
!    WRITE(*,FMT) (DBLE(z(i)), AIMAG(z(i)), i=1,3)
    Nsol=3
    DO i=1, Nsol
       sol(i)=DBLE(z(i))
       soli(i)=AIMAG(z(i))
    ENDDO

  ELSE IF(a(2) /= 0.0) THEN
    CALL QuadraticRoots(a, z)
!    WRITE(*,*) " Roots: REAL PART   IMAGINARY PART"
!    WRITE(*,FMT) (DBLE(z(i)), AIMAG(z(i)), i=1,2)
    Nsol=2
    DO i=1, Nsol
       sol(i)=DBLE(z(i))
       soli(i)=AIMAG(z(i))
    ENDDO

  ELSE IF(a(1) /= 0.0) THEN
    z(1)=CMPLX(-a(0)/a(1), 0.0, DP)      
!    WRITE(*,*) " Roots: REAL PART   IMAGINARY PART"
!    WRITE(*,FMT) (DBLE(z(i)), AIMAG(z(i)), i=1,1)
    Nsol=1
    DO i=1, Nsol
       sol(i)=DBLE(z(i))
       soli(i)=AIMAG(z(i))
    ENDDO

  ELSE
    Nsol=0
!    WRITE(*,*) "The equation is degenerate."
  END IF

!  STOP
END SUBROUTINE QUARTIC

SUBROUTINE SET_ANGLE(angle)
IMPLICIT NONE

REAL(DP)    :: angle

if (angle<0.) then
  do while (angle<0.) 
     angle=angle+TWPI
  enddo
else if (angle>=TWPI) then
  do while (angle>=TWPI) 
     angle=angle-TWPI
  enddo
endif
END SUBROUTINE SET_ANGLE

FUNCTION SETANGLE(anglein) RESULT(angleout)
IMPLICIT NONE

REAL(DP)    :: anglein, angleout

angleout=anglein

if (angleout<0.0D0) then
  do while (angleout<0.0D0) 
     angleout=angleout+TWPI
  enddo
else if (angleout>=TWPI) then
  do while (angleout>=TWPI) 
     angleout=angleout-TWPI
  enddo
endif

RETURN 
END FUNCTION SETANGLE

SUBROUTINE SOLVE_KEPLER_EQUATION(M,ecc,E)
IMPLICIT NONE
 
integer     :: i
real(DP)    :: M,ecc,E,PI

if (ecc.lt.0.8D0) then
     E=M
else
     E=PI
endif

do i=1, 10
     E=E-(E-ecc*sin(E)-M)/(1.-ecc*cos(E))
enddo
END SUBROUTINE SOLVE_KEPLER_EQUATION

SUBROUTINE NEWRAPS(x,y,dydx)
IMPLICIT NONE
 
REAL(DP)  ::  x,y,dydx,b

if (dydx.eq.0.0D0) then
    return
endif

b=y-dydx*x
x=-b/dydx

END SUBROUTINE NEWRAPS

SUBROUTINE INTEGRATE(starp,startmp,spotstmp,planetstmp,integral,status)
USE getInputs
IMPLICIT NONE

INTEGER            :: status
TYPE(startmp_)     :: startmp
TYPE(planetstmp_)  :: planetstmp
TYPE(spotstmp_)    :: spotstmp
TYPE(starparam_)   :: starp
REAL(DP)           :: integral

integral=0.0D0
status=0
call integral_star(starp,startmp)
call integral_spots(starp,spotstmp)
call integral_planets(starp,planetstmp)
integral=(startmp%integral_0-(startmp%integral+planetstmp%integral+spotstmp%integral))/startmp%integral_0
IF (integral/=integral) THEN
status=39
ENDIF

RETURN
END SUBROUTINE INTEGRATE

SUBROUTINE integral_star(starp,startmp)
USE getInputs
IMPLICIT NONE

TYPE(startmp_)    :: startmp
TYPE(starparam_)  :: starp
INTEGER           :: i
REAL(DP)          :: I_s_0,I_s_1,I_s_2,I_0,I_1,I_2,c1,c2

! Contribution of the unocculted star

I_s_0=I_star_0(dble(TWPI))-I_star_0(0.0d0)
I_s_1=I_star_1(dble(TWPI))-I_star_1(0.0d0)
I_s_2=I_star_2(dble(TWPI))-I_star_2(0.0d0)

! Contribution of additional arcs inside spots/planets

startmp%integral=0.0D0
I_0=0.0D0
I_1=0.0D0
I_2=0.0D0  

IF (startmp%nroots>0) THEN
 IF (startmp%nroots==1.AND.startmp%fgarc(1)==1) THEN
    I_0=I_0+startmp%arcffact(1)*(I_star_0(dble(TWPI))-I_star_0(0.0d0))
    I_1=I_1+startmp%arcffact(1)*(I_star_1(dble(TWPI))-I_star_1(0.0d0))
    I_2=I_2+startmp%arcffact(1)*(I_star_2(dble(TWPI))-I_star_2(0.0d0))     
 ELSE
    DO i=1, startmp%nroots-1
     IF (startmp%fgarc(i)==1) THEN      
       I_0=I_0+startmp%arcffact(i)*(I_star_0(startmp%roots(i+1))-I_star_0(startmp%roots(i)))
       I_1=I_1+startmp%arcffact(i)*(I_star_1(startmp%roots(i+1))-I_star_1(startmp%roots(i)))
       I_2=I_2+startmp%arcffact(i)*(I_star_2(startmp%roots(i+1))-I_star_2(startmp%roots(i)))
     ENDIF
    ENDDO
    IF (startmp%fgarc(startmp%nroots)==1) THEN      
    I_0=I_0+startmp%arcffact(startmp%nroots)* &
                ((I_star_0(dble(TWPI))-I_star_0(startmp%roots(startmp%nroots)))+ &
                                   (I_star_0(startmp%roots(1))-I_star_0(0.0d0)))
    I_1=I_1+startmp%arcffact(startmp%nroots)* & 
                ((I_star_1(dble(TWPI))-I_star_1(startmp%roots(startmp%nroots)))+ &
                                   (I_star_1(startmp%roots(1))-I_star_1(0.0d0)))
    I_2=I_2+startmp%arcffact(startmp%nroots)* &
                ((I_star_2(dble(TWPI))-I_star_2(startmp%roots(startmp%nroots)))+ &
                                   (I_star_2(startmp%roots(1))-I_star_2(0.0d0)))
    ENDIF
 ENDIF
ENDIF

c1=starp%c1
c2=starp%c2
startmp%integral_0=(I_s_0)*(1.-c1-2.*c2)+(I_s_1)*(c1+2.*c2)+(I_s_2)*(c2)
startmp%integral=(I_0)*(1.-c1-2.*c2)+(I_1)*(c1+2.*c2)+(I_2)*(c2)

END SUBROUTINE integral_star

FUNCTION I_star_0(beta)
IMPLICIT NONE

REAL(DP)   :: beta, I_star_0

I_star_0=0.5d0*(beta+sin(beta)*cos(beta))

RETURN
END FUNCTION I_star_0

FUNCTION I_star_1(beta)
IMPLICIT NONE

REAL(DP)   :: beta, I_star_1

I_star_1=(1.0d0/3.0d0)*(beta)
RETURN
END FUNCTION I_star_1

FUNCTION I_star_2(beta)
IMPLICIT NONE

REAL(DP)   :: beta, I_star_2

I_star_2=1.0D0/48.0d0*((12.0d0)*beta-sin(4.0d0*beta))

RETURN
END FUNCTION I_star_2

SUBROUTINE integral_planets(starp,planetstmp)
USE getInputs
IMPLICIT NONE

TYPE(planetstmp_)  :: planetstmp
TYPE(plttmp_)      :: plttmp1, plttmp02
TYPE(starparam_)   :: starp
INTEGER            :: i,k
REAL(DP)           :: I_0,I_1,I_2,c1,c2

I_0=0.0D0
I_1=0.0D0
I_2=0.0D0  
planetstmp%integral=0.0D0

IF (planetstmp%nplanets>0) THEN
CALL setroots(planetstmp,plttmp02,plttmp1)
DO i=1, planetstmp%nplanets

IF (planetstmp%nroots(i)>0) THEN

 IF (plttmp02%nroots(i)==1.AND.plttmp02%fgarc(i,1)==1) THEN
    I_0=I_0+plttmp02%arcffact(i,1)*(I_planets_0(i,dble(TWPI),planetstmp)-I_planets_0(i,0.0D0,planetstmp))
    I_2=I_2+plttmp02%arcffact(i,1)*(I_planets_2(i,dble(TWPI),planetstmp)-I_planets_2(i,0.0D0,planetstmp))
 ELSE
    DO k=1, plttmp02%nroots(i)-1
     IF (plttmp02%fgarc(i,k)==1) THEN      
       I_0=I_0+plttmp02%arcffact(i,k)*(I_planets_0(i,plttmp02%roots(i,k+1),planetstmp)- & 
                   I_planets_0(i,plttmp02%roots(i,k),planetstmp))
       I_2=I_2+plttmp02%arcffact(i,k)*(I_planets_2(i,plttmp02%roots(i,k+1),planetstmp)- & 
                   I_planets_2(i,plttmp02%roots(i,k),planetstmp))
     ENDIF
    ENDDO
    IF (plttmp02%fgarc(i,plttmp02%nroots(i))==1) THEN      
    I_0=I_0+plttmp02%arcffact(i,plttmp02%nroots(i))* &
       ((I_planets_0(i,dble(TWPI),planetstmp)- & 
         I_planets_0(i,plttmp02%roots(i,plttmp02%nroots(i)),planetstmp))+ &
        (I_planets_0(i,plttmp02%roots(i,1),planetstmp)-I_planets_0(i,0.0d0,planetstmp)))
    I_2=I_2+plttmp02%arcffact(i,plttmp02%nroots(i))* &
       ((I_planets_2(i,dble(TWPI),planetstmp)- &
         I_planets_2(i,plttmp02%roots(i,plttmp02%nroots(i)),planetstmp))+ &
        (I_planets_2(i,plttmp02%roots(i,1),planetstmp)-I_planets_2(i,0.0d0,planetstmp)))   
    ENDIF
 ENDIF

 IF (plttmp1%nroots(i)==1.AND.plttmp1%fgarc(i,1)==1) THEN
    I_1=I_1+plttmp1%arcffact(i,1)*(I_planets_1(i,dble(TWPI),planetstmp)-I_planets_1(i,0.0D0,planetstmp))
 ELSE
    DO k=1, plttmp1%nroots(i)-1
     IF (plttmp1%fgarc(i,k)==1) THEN      
       I_1=I_1+plttmp1%arcffact(i,k)*(I_planets_1(i,plttmp1%roots(i,k+1),planetstmp)- & 
                   I_planets_1(i,plttmp1%roots(i,k),planetstmp))
     ENDIF
    ENDDO
    IF (plttmp1%fgarc(i,plttmp1%nroots(i))==1) THEN      
      I_1=I_1+plttmp1%arcffact(i,plttmp1%nroots(i))* & 
       ((I_planets_1(i,dble(TWPI),planetstmp)- & 
         I_planets_1(i,plttmp1%roots(i,plttmp1%nroots(i)),planetstmp))+ &
        (I_planets_1(i,plttmp1%roots(i,1),planetstmp)-I_planets_1(i,0.0d0,planetstmp)))
    ENDIF
 ENDIF

ENDIF

ENDDO
ENDIF

c1=starp%c1
c2=starp%c2
planetstmp%integral=(I_0)*(1.-c1-2.*c2)+(I_1)*(c1+2.*c2)+(I_2)*(c2)

IF (ASSOCIATED(plttmp02%roots)) DEALLOCATE(plttmp02%roots)
IF (ASSOCIATED(plttmp1%roots))  DEALLOCATE(plttmp1%roots)
IF (ASSOCIATED(plttmp02%arcffact)) DEALLOCATE(plttmp02%arcffact)
IF (ASSOCIATED(plttmp1%arcffact))  DEALLOCATE(plttmp1%arcffact)
IF (ASSOCIATED(plttmp02%fgarc)) DEALLOCATE(plttmp02%fgarc)
IF (ASSOCIATED(plttmp1%fgarc))  DEALLOCATE(plttmp1%fgarc)
IF (ASSOCIATED(plttmp02%nroots)) DEALLOCATE(plttmp02%nroots)
IF (ASSOCIATED(plttmp1%nroots))  DEALLOCATE(plttmp1%nroots)

END SUBROUTINE integral_planets

SUBROUTINE setroots(planetstmp,plttmp02,plttmp1)
IMPLICIT NONE

INTEGER                   :: i, k, kk, status
TYPE(planetstmp_)         :: planetstmp
TYPE(plttmp_)             :: plttmp02, plttmp1
INTEGER, DIMENSION(2)     :: shpr
REAL(DP)                  :: arcffact_,roots_,fgarc_
REAL(DP)                  :: x0, y0, angle

IF (ASSOCIATED(plttmp02%roots)) DEALLOCATE(plttmp02%roots)
IF (ASSOCIATED(plttmp1%roots))  DEALLOCATE(plttmp1%roots)
IF (ASSOCIATED(plttmp02%arcffact)) DEALLOCATE(plttmp02%arcffact)
IF (ASSOCIATED(plttmp1%arcffact))  DEALLOCATE(plttmp1%arcffact)
IF (ASSOCIATED(plttmp02%fgarc)) DEALLOCATE(plttmp02%fgarc)
IF (ASSOCIATED(plttmp1%fgarc))  DEALLOCATE(plttmp1%fgarc)
IF (ASSOCIATED(plttmp02%nroots)) DEALLOCATE(plttmp02%nroots)
IF (ASSOCIATED(plttmp1%nroots))  DEALLOCATE(plttmp1%nroots)

shpr=SHAPE(planetstmp%roots)

ALLOCATE(plttmp02%roots(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)
ALLOCATE(plttmp1%roots(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)

ALLOCATE(plttmp02%arcffact(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)
ALLOCATE(plttmp1%arcffact(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)

ALLOCATE(plttmp02%fgarc(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)
ALLOCATE(plttmp1%fgarc(shpr(1),shpr(2)), STAT=status)
IF (status/=0) call errmsg(35)

ALLOCATE(plttmp02%nroots(shpr(1)), STAT=status)
IF (status/=0) call errmsg(35)
ALLOCATE(plttmp1%nroots(shpr(1)), STAT=status)
IF (status/=0) call errmsg(35)

plttmp02%roots=planetstmp%roots
plttmp02%arcffact=planetstmp%arcffact
plttmp02%fgarc=planetstmp%fgarc
plttmp02%nroots=planetstmp%nroots

plttmp1%roots=planetstmp%roots
plttmp1%arcffact=planetstmp%arcffact
plttmp1%fgarc=planetstmp%fgarc     
plttmp1%nroots=planetstmp%nroots

! Apply rotation angles

DO i=1, planetstmp%nplanets
   IF (planetstmp%nroots(i)>1) THEN
     angle=setangle(planetstmp%phi(i)+HPI)
     x0=planetstmp%y(i)*cos(angle)
     y0=planetstmp%y(i)*sin(angle)
     DO k=1, planetstmp%nroots(i)
          plttmp02%roots(i,k)=setangle(planetstmp%roots(i,k)+planetstmp%phi(i))
          plttmp1%roots(i,k)=setangle(planetstmp%roots(i,k)+planetstmp%phi(i)-atan2(y0,x0))
     ENDDO
   ENDIF
ENDDO

! Resort angles and all the correspondent vectors

DO i=1, planetstmp%nplanets
   IF (planetstmp%nroots(i)>1) THEN
    DO k=1, planetstmp%nroots(i)-1
       DO kk=k+1, planetstmp%nroots(i)   
  
          IF (plttmp02%roots(i,kk)<plttmp02%roots(i,k)) THEN
               roots_=plttmp02%roots(i,kk)
               plttmp02%roots(i,kk)=plttmp02%roots(i,k)
               plttmp02%roots(i,k)=roots_

               arcffact_=plttmp02%arcffact(i,kk)
               plttmp02%arcffact(i,kk)=plttmp02%arcffact(i,k)
               plttmp02%arcffact(i,k)=arcffact_

               fgarc_=plttmp02%fgarc(i,kk)
               plttmp02%fgarc(i,kk)=plttmp02%fgarc(i,k)
               plttmp02%fgarc(i,k)=fgarc_
          ENDIF        

          IF (plttmp1%roots(i,kk)<plttmp1%roots(i,k)) THEN
               roots_=plttmp1%roots(i,kk)
               plttmp1%roots(i,kk)=plttmp1%roots(i,k)
               plttmp1%roots(i,k)=roots_

               arcffact_=plttmp1%arcffact(i,kk)
               plttmp1%arcffact(i,kk)=plttmp1%arcffact(i,k)
               plttmp1%arcffact(i,k)=arcffact_

               fgarc_=plttmp1%fgarc(i,kk)
               plttmp1%fgarc(i,kk)=plttmp1%fgarc(i,k)
               plttmp1%fgarc(i,k)=fgarc_
          ENDIF        

       ENDDO 
    ENDDO
   ENDIF
ENDDO

END SUBROUTINE setroots

FUNCTION I_planets_0(i,psi,planetstmp)
IMPLICIT NONE

INTEGER  :: i
REAL(DP) :: angle,psi,r,d,I_planets_0
TYPE(planetstmp_)  :: planetstmp

r=planetstmp%dim(i)
d=planetstmp%y(i)
angle=setangle(planetstmp%phi(i)+HPI)


I_planets_0=(0.5d0*(psi+sin(psi)*cos(psi))* &
                   (r**2)+r*d*cos(angle)*sin(psi))

RETURN
END FUNCTION I_planets_0

FUNCTION I_planets_1(i,psi,planetstmp)
IMPLICIT NONE

INTEGER  :: i,fg
REAL(DP) :: angle, psi, iret
REAL(DP) :: r,c,q2,d2,s2,cx,sx,rf_,rd_,rj_,w
REAL(DP) :: x,xx,beta,I_planets_1,x0,y0
TYPE(planetstmp_) :: planetstmp

iret=0.0D0

r=planetstmp%dim(i)
c=planetstmp%y(i)

IF (psi.eq.TWPI) then
   x=PI
   fg=0
ELSE IF (psi/=TWPI) then
   x=psi
   fg=1
ENDIF

55 CONTINUE
q2=r*r+c*c+2.0D0*r*c*cos(x)
d2=r*r+c*c-2.0D0*r*c
s2=r*r+c*c+2.0D0*r*c

sx=sin(x/2.0D0)
cx=cos(x/2.0D0)

if (1.0D0.lt.q2) q2=1.0D0

w=(1.0D0-q2)/(1.0D0-d2)
if (w.lt.0.0D0) w=0.00D0
rf_=rf(w,sx*sx,1.0D0)
rd_=rd(w,sx*sx,1.0D0)
if (r.ne.c)  then 
    rj_=rj(w,sx*sx,1.0D0,q2/d2)
else           
    rj_=0.0
endif

beta=atan2((c-r)*sx,(c+r)*cx)
iret=iret-beta/3.0D0
iret=iret+x/6.0D0

w=cx/sqrt(1.0D0-d2)

iret=iret+2.0D0/ 9.0D0*c*r*sin(x)*sqrt(1.0D0-q2)+ &
          1.0D0/ 3.0D0*(1.0D0+2.0D0*r*r*r*r-4.0D0*r*r)*w*rf_+ &
          2.0D0/ 9.0D0*r*c*(4.0D0-7.0D0*r*r-c*c+5.0D0*r*c)*w*rf_- &
          4.0D0/27.0D0*r*c*(4.0D0-7.0D0*r*r-c*c)*w*cx*cx*rd_ 
if (r.ne.c) then
    iret = iret + 1.0D0/3.0D0*w*(r+c)/(r-c)*(rf_-(q2-d2)/(3.0D0*d2)*rj_)
else
    iret = iret - 1.0D0/3.0D0*w*(r+c)*(q2-d2)*PI/(2.0D0*q2*sqrt(q2))
endif

IF ((psi.EQ.TWPI).AND.(fg==0)) THEN
     I_planets_1=2.0D0*iret
     x=0.0D0
     iret=0.0d0
     fg=1
     GO TO 55
ELSE IF ((psi.EQ.TWPI).AND.(fg==1)) THEN
     I_planets_1=I_planets_1-iret
ELSE IF (psi/=TWPI) THEN
     I_planets_1=iret
ENDIF

RETURN
END FUNCTION I_planets_1

FUNCTION I_planets_2(i,psi,planetstmp)
IMPLICIT NONE

INTEGER :: i
REAL(DP)  :: I_planets_2,angle,r,x0,y0,psi
TYPE(planetstmp_)  :: planetstmp

r=planetstmp%dim(i)
angle=setangle(planetstmp%phi(i)+HPI)
x0=planetstmp%y(i)*cos(angle)
y0=planetstmp%y(i)*sin(angle)

I_planets_2=r/48.0d0* &
         ((24.0d0*(x0**2+y0**2)*r+12.0d0*r**3)*psi- &
           4.0d0*y0*(6.0d0*x0**2+2.0d0*y0**2+9.0d0*r**2)*cos(psi)+ &
           4.0d0*x0*(2.0d0*x0**2+6.0d0*y0**2+9.0d0*r**2)*sin(psi)- &
           4.0d0*r**2*(y0*cos(3.0d0*psi)+x0*sin(3.0d0*psi))- &
           24.0d0*x0*y0*r*cos(2.0d0*psi)-(r**3)*sin(4.0d0*psi))

RETURN
END FUNCTION I_planets_2

SUBROUTINE integral_spots(starp,spotstmp)
USE getInputs
IMPLICIT NONE

TYPE(spotstmp_)    :: spotstmp
TYPE(starparam_)   :: starp
INTEGER            :: i,k
REAL(DP)           :: I_0,I_1,I_2,c1,c2,A(14)

I_0=0.0D0
I_1=0.0D0
I_2=0.0D0  
spotstmp%integral=0.0D0

IF (spotstmp%nspots>0) THEN
DO i=1, spotstmp%nspots

IF (spotstmp%nroots(i)>0) THEN
 CALL I_spots_2_fixed(i,spotstmp,A)
 IF (spotstmp%nroots(i)==1.AND.spotstmp%fgarc(i,1)==1) THEN

    I_0=I_0+spotstmp%arcffact(i,1)*(I_spots_0(i,dble(TWPI),spotstmp)-I_spots_0(i,0.0d0,spotstmp))
    I_1=I_1+spotstmp%arcffact(i,1)*(I_spots_1(i,dble(TWPI),spotstmp)-I_spots_1(i,0.0d0,spotstmp))
    I_2=I_2+spotstmp%arcffact(i,1)*(I_spots_2(i,dble(TWPI),A,spotstmp)-I_spots_2(i,0.0d0,A,spotstmp))

 ELSE
    DO k=1, spotstmp%nroots(i)-1
     IF (spotstmp%fgarc(i,k)==1) THEN
       I_0=I_0+spotstmp%arcffact(i,k)*(I_spots_0(i,spotstmp%roots(i,k+1),spotstmp)- & 
           I_spots_0(i,spotstmp%roots(i,k),spotstmp))
       I_1=I_1+spotstmp%arcffact(i,k)*(I_spots_1(i,spotstmp%roots(i,k+1),spotstmp)- & 
           I_spots_1(i,spotstmp%roots(i,k),spotstmp))
       I_2=I_2+spotstmp%arcffact(i,k)*(I_spots_2(i,spotstmp%roots(i,k+1),A,spotstmp)- & 
           I_spots_2(i,spotstmp%roots(i,k),A,spotstmp))
     ENDIF
    ENDDO
    IF (spotstmp%fgarc(i,spotstmp%nroots(i))==1) THEN      
    I_0=I_0+spotstmp%arcffact(i,spotstmp%nroots(i))* &
       ((I_spots_0(i,dble(TWPI),spotstmp)-I_spots_0(i,spotstmp%roots(i,spotstmp%nroots(i)),spotstmp))+ &
                (I_spots_0(i,spotstmp%roots(i,1),spotstmp)-I_spots_0(i,0.0d0,spotstmp)))
    I_1=I_1+spotstmp%arcffact(i,spotstmp%nroots(i))* & 
       ((I_spots_1(i,dble(TWPI),spotstmp)-I_spots_1(i,spotstmp%roots(i,spotstmp%nroots(i)),spotstmp))+ &
                (I_spots_1(i,spotstmp%roots(i,1),spotstmp)-I_spots_1(i,0.0d0,spotstmp)))
    I_2=I_2+spotstmp%arcffact(i,spotstmp%nroots(i))* &
       ((I_spots_2(i,dble(TWPI),A,spotstmp)-I_spots_2(i,spotstmp%roots(i,spotstmp%nroots(i)),A,spotstmp))+ &
                (I_spots_2(i,spotstmp%roots(i,1),A,spotstmp)-I_spots_2(i,0.0d0,A,spotstmp)))
    ENDIF
 ENDIF
ENDIF

ENDDO
ENDIF

c1=starp%c1
c2=starp%c2
spotstmp%integral=(I_0)*(1.-c1-2.*c2)+(I_1)*(c1+2.*c2)+(I_2)*(c2)

END SUBROUTINE integral_spots

FUNCTION I_spots_0(i,alpha,spotstmp)
IMPLICIT NONE

INTEGER   :: i
real(DP)  :: alpha,theta,psi
real(DP)  :: phi,PI,I_spots_0
TYPE(spotstmp_) :: spotstmp

phi=setangle(-spotstmp%phi(i))
theta=spotstmp%dim(i)
psi=spotstmp%y(i)

I_spots_0=1.0D0/8.0D0 * sin(theta)**2 * sin(2.0D0*phi) * &
                 cos(2.0D0*alpha) * (1.0D0 + cos(psi)**2)+ &
                 0.5D0 * cos(psi) * sin(theta)**2 * (alpha - &
                 0.5D0 * sin(2.0D0*alpha) * cos(2.0D0*phi)) - &
                 0.5D0*sin(psi)*sin(2.0D0*theta)*sin(phi)**2*sin(alpha)- &
                 1.0D0/8.0D0*sin(2.0D0*psi)*sin(2.0D0*theta)*sin(2.0D0*phi)*cos(alpha)


RETURN
END FUNCTION I_spots_0

FUNCTION I_spots_1(i,alpha,spotstmp)
IMPLICIT NONE

INTEGER :: i
REAL(DP) :: t,p,phi,a,z,a_,b_,c_,d_
REAL(DP) :: I_spots_1,I_spots_1_PI,sgn,alpha
TYPE(spotstmp_) :: spotstmp

      t=spotstmp%dim(i)
      p=spotstmp%y(i)
      phi=setangle(-spotstmp%phi(i))
      a=alpha
      z=cos(p)*cos(t)+sin(p)*sin(t)*cos(a)

      a_=cos(p)*cos(t)
      b_=sin(p)*sin(t)
      c_=1./4.*sin(2.*p)*sin(2.*t)
      d_=-1./2.*cos(p)**2 * sin(t)**2

      I_spots_1=0.0D0
      I_spots_1_PI=0.0D0

      if ((t+p).gt.PI) then 
          sgn=-1.0D0
      else if ((t+p).eq.PI) then
          sgn=0.0D0
      else
          sgn=1.0D0
      endif

      if (a.gt.PI) then

       I_spots_1_PI=1./3.*(-1./32. * PI * &
              (3. * cos(2.*p-3.*t) - 8. * cos(p-2.*t) - &
               3. * cos(2.*p-t)    - 3. * cos(2.*p+t) - &
               8. * cos(p+2.*t)    + 3. * cos(2.*p+3.*t) + &
              16. * cos(p) + 30. * cos(t) + 2. * cos(3.*t)) + &
              sgn * PI + cos(p)*(sin(t)**2)*PI)

       a=2.*PI-a

       I_spots_1=1./3. * &
           (-1./8. * (sin(p)**2) * sin(2.*t) * sin(2.*a) * sin(t) - &
             1./2. *  sin(p) * sin(a) * sin(t) * &
             (cos(p-2.*t) + cos(p+2.*t) - 2. * cos(t)) - &
            1./32. * a * &
              (3. * cos(2.*p-3.*t) - 8. * cos(p-2.*t) - &
               3. * cos(2.*p-t)    - 3. * cos(2.*p+t) - &
               8. * cos(p+2.*t)    + 3. * cos(2.*p+3.*t) + &
              16. * cos(p) + 30. * cos(t) + 2. * cos(3.*t)) + &
               2. * &
            atan2(sin(a/2.)*cos((t+p)/2.),cos(a/2.)*cos((p-t)/2.))+ &
            cos(p) * (sin(t)**2) * a - &
            sin(p) * cos(t) * sin(t) * sin(a))

       I_spots_1=2.*I_spots_1_PI-I_spots_1

      else if (a.eq.PI) then

       I_spots_1=1./3.* (-1./32. * PI * &
              (3. * cos(2.*p-3.*t) - 8. * cos(p-2.*t) - &
               3. * cos(2.*p-t)    - 3. * cos(2.*p+t) - &
               8. * cos(p+2.*t)    + 3. * cos(2.*p+3.*t) + &
              16. * cos(p) + 30. * cos(t) + 2. * cos(3.*t)) + &
              sgn * PI + cos(p)*(sin(t)**2)*PI)

      else

       I_spots_1=1./3. * &
           (-1./8. * (sin(p)**2) * sin(2.*t) * sin(2.*a) * sin(t) - &
             1./2. *  sin(p) * sin(a) * sin(t) * &
             (cos(p-2.*t) + cos(p+2.*t) - 2. * cos(t)) - &
            1./32. * a * &
              (3. * cos(2.*p-3.*t) - 8. * cos(p-2.*t) - &
               3. * cos(2.*p-t)    - 3. * cos(2.*p+t) - &
               8. * cos(p+2.*t)    + 3. * cos(2.*p+3.*t) + &
              16. * cos(p) + 30. * cos(t) + 2. * cos(3.*t)) + &
               2. * &
            atan2(sin(a/2.)*cos((t+p)/2.),cos(a/2.)*cos((p-t)/2.))+ &
            cos(p) * (sin(t)**2) * a - &
            sin(p) * cos(t) * sin(t) * sin(a))

      endif

RETURN
END FUNCTION I_spots_1

SUBROUTINE I_spots_2_fixed(i,spotstmp,A)
IMPLICIT NONE

INTEGER  ::  i,j
real(DP) ::  t,p
real(DP), INTENT(OUT) ::  A(14)
real(DP) ::  q,m,n,phi,I_spots_2
TYPE(spotstmp_)  :: spotstmp

      t=spotstmp%dim(i)
      p=spotstmp%y(i)
      phi=setangle(-spotstmp%phi(i))

      q=sin(2.*phi) * cos(2.*phi) / 2.
      m=-cos(phi)**2 * sin(phi)**2+0.5*(cos(phi)**4 + sin(phi)**4)
      n=cos(phi)**2 * sin(phi)**2+1./6.*(cos(phi)**4 + sin(phi)**4)

      A(1)=-(0.5+cos(p)**2)*(sin(t)**3 * cos(t) * sin(p)) * 2. * m
      A(2)=(0.5 * sin(t)**4 * cos(p)) * (1.+cos(p)**2) * 2. * m
      A(3)=0.5 * sin(t)**2 * cos(t)**2 * cos(p) * sin(p)**2 * 2.* m
      A(4)=-1./6. * cos(t)**3 * sin(t) * sin(p)**3 * 6.* n
      A(5)=1./2. * cos(t)**2 * sin(t)**2 * sin(p)**2 * cos(p) * 6.* n
      A(6)=-1./2. * cos(t) * sin(t)**3 * sin(p) * cos(p)**2 * 6.* n
      A(7)=1./6. * sin(t)**4 * cos(p)**3 * 6.* n
      A(8)=1./6. * sin(t)**4 * cos(p) * 6.* n
      A(9)=q/3. * sin(t)**4 * (1.+3.*cos(p)**2)
      A(10)=q/3. * cos(t)**3 * sin(t) * cos(p) * sin(p)**3
      A(11)=-q * cos(t)**2 * sin(t)**2 * sin(p)**2 * (cos(p)**2+1.)
      A(12)=q * cos(t) * sin(t)**3 * sin(p) * cos(p) * (cos(p)**2+2.)
      A(13)=q/3. * cos(p)**2 * sin(t)**4 * (cos(p)**2+3.)
      A(14)=-q*cos(p)*sin(p)*cos(t)*sin(t)**3


RETURN
END SUBROUTINE I_spots_2_fixed

FUNCTION I_spots_2(i,alpha,A,spotstmp)
IMPLICIT NONE

INTEGER  ::  i,j
real(DP) ::  alpha,aa,t,p
real(DP), INTENT(IN) ::  A(14)
real(DP) ::  B(14)
real(DP) ::  q,m,n,phi,I_spots_2
TYPE(spotstmp_)  :: spotstmp

      aa=alpha

      B(1)=1./3.*sin(aa)**3
      B(2)=1./32.*(4*aa-sin(4.*aa))
      B(3)=1./2.*(aa - sin(aa) * cos(aa))
      B(4)=sin(aa)
      B(5)=0.5*(aa+sin(aa)*cos(aa))
      B(6)=1./12.*(9.*sin(aa)+sin(3.*aa))
      B(7)=1./32.*(12.*aa+8.*sin(2.*aa)+sin(4.*aa))
      B(8)=1./32.*(12.*aa-8.*sin(2.*aa)+sin(4.*aa))
      B(9)=sin(aa)**4/4.
      B(10)=-cos(aa)
      B(11)=0.5*sin(aa)**2
      B(12)=-1./3.*cos(aa)**3
      B(13)=-cos(aa)**4/4.
      B(14)=1./12*(cos(3.*aa)-9.*cos(aa))

      I_spots_2=DOT_PRODUCT(A,B)

RETURN
END FUNCTION I_spots_2

!  (C) Copr. 1986-92 Numerical Recipes Software ,2:-5K#R..
FUNCTION rc(x,y)
IMPLICIT NONE

REAL(DP) :: rc,x,y 
REAL(DP), PARAMETER :: ERRTOL=.04,TINY=1.69e-38,SQRTNY=1.3e-19,BIG=3.E37
REAL(DP), PARAMETER :: TNBG=TINY*BIG,COMP1=2.236/SQRTNY,COMP2=TNBG*TNBG/25.,THIRD=1./3.
REAL(DP), PARAMETER :: C1=0.3D0,C2=1.0D0/7.0D0,C3=0.375D0,C4=9.0D0/22.0D0
REAL(DP) :: alamb,ave,s,w,xt,yt

      if(x.lt.0.0D0.or.y.eq.0.0D0.or.(x+abs(y)).lt.TINY.or.(x+ &
      abs(y)).gt.BIG.or.(y.lt.-COMP1.and.x.gt.0.0D0.and.x.lt.COMP2)) THEN
         print*,'invalid arguments in rc' 
         return
      ENDIF
      if(y.gt.0.0D0)then
        xt=x
        yt=y
        w=1.0D0
      else
        xt=x-y
        yt=-y
        w=sqrt(x)/sqrt(xt)
      endif
1     continue
        alamb=2.0D0*sqrt(xt)*sqrt(yt)+yt
        xt=0.25D0*(xt+alamb)
        yt=0.25D0*(yt+alamb)
        ave=THIRD*(xt+yt+yt)
        s=(yt-ave)/ave
      if(abs(s).gt.ERRTOL)goto 1
      rc=w*(1.0D0+s*s*(C1+s*(C2+s*(C3+s*C4))))/sqrt(ave)
RETURN
END FUNCTION rc

!  (C) Copr. 1986-92 Numerical Recipes Software ,2:-5K#R..
      FUNCTION rf(x,y,z)
      IMPLICIT NONE
      REAL(DP) :: rf,x,y,z 
      REAL(DP), PARAMETER :: ERRTOL=.08,TINY=1.5e-38,BIG=3.E37,THIRD=1./3.
      REAL(DP), PARAMETER :: C1=1./24.,C2=.1,C3=3./44.,C4=1./14.
      REAL(DP) :: alamb,ave,delx,dely,delz,e2,e3,sqrtx,sqrty,sqrtz,xt,yt,zt
      if(min(x,y,z).lt.0.0D0.or.min(x+y,x+z,y+z).lt.TINY.or.max(x,y, &
      z).gt.BIG) then 
        print*,'invalid arguments in rf'
        return
      ENDIF
      xt=x
      yt=y
      zt=z
1     continue
        sqrtx=sqrt(xt)
        sqrty=sqrt(yt)
        sqrtz=sqrt(zt)
        alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz
        xt=0.25D0*(xt+alamb)
        yt=0.25D0*(yt+alamb)
        zt=0.25D0*(zt+alamb)
        ave=THIRD*(xt+yt+zt)
        delx=(ave-xt)/ave
        dely=(ave-yt)/ave
        delz=(ave-zt)/ave
      if(max(abs(delx),abs(dely),abs(delz)).gt.ERRTOL)goto 1
      e2=delx*dely-delz**2
      e3=delx*dely*delz
      rf=(1.0D0+(C1*e2-C2-C3*e3)*e2+C4*e3)/sqrt(ave)
      return
      END FUNCTION rf

!  (C) Copr. 1986-92 Numerical Recipes Software ,2:-5K#R..
      FUNCTION rd(x,y,z)
      IMPLICIT NONE
      REAL(DP) :: rd,x,y,z !,ERRTOL,TINY,BIG,C1,C2,C3,C4,C5,C6
      REAL(DP), PARAMETER :: ERRTOL=0.05D0,TINY=1.e-25,BIG=4.5E21,C1=3.0D0/14.0D0,C2=1.0D0/6.0D0
      REAL(DP), PARAMETER :: C3=9.0D0/22.0D0,C4=3.0D0/26.0D0,C5=0.25D0*C3,C6=1.5D0*C4
      REAL(DP) :: alamb,ave,delx,dely,delz,ea,eb,ec,ed,ee,fac,sqrtx,sqrty, &
      sqrtz,sum,xt,yt,zt
      if(min(x,y).lt.0.0D0.or.min(x+y,z).lt.TINY.or.max(x,y,z).gt.BIG) THEN 
        print*,'invalid arguments in rd'
        return
      ENDIF
      xt=x
      yt=y
      zt=z
      sum=0.0D0
      fac=1.0D0
1     continue
        sqrtx=sqrt(xt)
        sqrty=sqrt(yt)
        sqrtz=sqrt(zt)
        alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz
        sum=sum+fac/(sqrtz*(zt+alamb))
        fac=0.25D0*fac
        xt=0.25D0*(xt+alamb)
        yt=0.25D0*(yt+alamb)
        zt=0.25D0*(zt+alamb)
        ave=0.2D0*(xt+yt+3.*zt)
        delx=(ave-xt)/ave
        dely=(ave-yt)/ave
        delz=(ave-zt)/ave
      if(max(abs(delx),abs(dely),abs(delz)).gt.ERRTOL)goto 1
      ea=delx*dely
      eb=delz*delz
      ec=ea-eb
      ed=ea-6.0D0*eb
      ee=ed+ec+ec
      rd=3.0D0*sum+fac*(1.0D0+ed*(-C1+C5*ed-C6*delz*ee)+delz*(C2*ee+delz*(-C3* &
      ec+delz*C4*ea)))/(ave*sqrt(ave))
      return
      END FUNCTION rd

!  (C) Copr. 1986-92 Numerical Recipes Software ,2:-5K#R..
      FUNCTION rj(x,y,z,p)
      IMPLICIT NONE
      REAL(DP) :: rj,p,x,y,z 
      REAL(DP), PARAMETER :: ERRTOL=.05,TINY=2.5e-13,BIG=9.E11,C1=3./14.,C2=1./3.
      REAL(DP), PARAMETER :: C3=3.0D0/22.0D0,C4=3.0D0/26.0D0
      REAL(DP), PARAMETER :: C5=0.75D0*C3,C6=1.5D0*C4,C7=0.5D0*C2,C8=C3+C3
!     USES rc,rf
      REAL(DP) :: a,alamb,alpha,ave,b,beta,delp,delx,dely,delz,ea,eb,ec,ed,ee
      REAL(DP) :: fac,pt,rcx,rho,sqrtx,sqrty,sqrtz,sum,tau,xt,yt,zt

      if(min(x,y,z).lt.0.0D0.or.min(x+y,x+z,y+z,abs(p)).lt.TINY.or.max(x,y, &
      z,abs(p)).gt.BIG) THEN
         print*,'invalid arguments in rj'
         return
      ENDIF
      sum=0.0D0
      fac=1.0D0
      if(p.gt.0.0D0)then
        xt=x
        yt=y
        zt=z
        pt=p
      else
        xt=min(x,y,z)
        zt=max(x,y,z)
        yt=x+y+z-xt-zt
        a=1./(yt-p)
        b=a*(zt-yt)*(yt-xt)
        pt=yt+b
        rho=xt*zt/yt
        tau=p*pt/yt
        rcx=rc(rho,tau)
      endif
1     continue
        sqrtx=sqrt(xt)
        sqrty=sqrt(yt)
        sqrtz=sqrt(zt)
        alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz
        alpha=(pt*(sqrtx+sqrty+sqrtz)+sqrtx*sqrty*sqrtz)**2
        beta=pt*(pt+alamb)**2
        sum=sum+fac*rc(alpha,beta)
        fac=0.25D0*fac
        xt=0.25D0*(xt+alamb)
        yt=0.25D0*(yt+alamb)
        zt=0.25D0*(zt+alamb)
        pt=0.25D0*(pt+alamb)
        ave=0.2D0*(xt+yt+zt+pt+pt)
        delx=(ave-xt)/ave
        dely=(ave-yt)/ave
        delz=(ave-zt)/ave
        delp=(ave-pt)/ave
      if(max(abs(delx),abs(dely),abs(delz),abs(delp)).gt.ERRTOL)goto 1
      ea=delx*(dely+delz)+dely*delz
      eb=delx*dely*delz
      ec=delp**2
      ed=ea-3.*ec
      ee=eb+2.*delp*(ea-ec)
      rj=3.*sum+fac*(1.+ed*(-C1+C5*ed-C6*ee)+eb*(C7+delp*(-C8+delp*C4))+ &
      delp*ea*(C2-delp*C3)-C2*delp*ec)/(ave*sqrt(ave))
      if (p.le.0.0D0) rj=a*(b*rj+3.0D0*(rcx-rf(xt,yt,zt)))
      return
      END FUNCTION rj

SUBROUTINE DEALLOCATE_TEMPORARY_VECTORS(startmp,spotstmp,planetstmp)
IMPLICIT NONE

TYPE(startmp_) :: startmp
TYPE(spotstmp_) :: spotstmp
TYPE(planetstmp_) :: planetstmp

IF (ASSOCIATED(startmp%roots)) DEALLOCATE (startmp%roots)
IF (ASSOCIATED(startmp%fgarc)) DEALLOCATE (startmp%fgarc)
IF (ASSOCIATED(startmp%midangle)) DEALLOCATE (startmp%midangle)
IF (ASSOCIATED(startmp%sumffact)) DEALLOCATE (startmp%sumffact)
IF (ASSOCIATED(startmp%arcffact)) DEALLOCATE (startmp%arcffact)
IF (ASSOCIATED(startmp%tgpoint)) DEALLOCATE (startmp%tgpoint)

IF (ASSOCIATED(spotstmp%id)) DEALLOCATE(spotstmp%id)
IF (ASSOCIATED(spotstmp%y)) DEALLOCATE(spotstmp%y)
IF (ASSOCIATED(spotstmp%phi)) DEALLOCATE(spotstmp%phi)
IF (ASSOCIATED(spotstmp%filter)) DEALLOCATE(spotstmp%filter)
IF (ASSOCIATED(spotstmp%dim)) DEALLOCATE(spotstmp%dim)
IF (ASSOCIATED(spotstmp%ffact)) DEALLOCATE(spotstmp%ffact)
IF (ASSOCIATED(spotstmp%roots)) DEALLOCATE (spotstmp%roots)
IF (ASSOCIATED(spotstmp%fgarc)) DEALLOCATE (spotstmp%fgarc)
IF (ASSOCIATED(spotstmp%nroots)) DEALLOCATE (spotstmp%nroots)
IF (ASSOCIATED(spotstmp%midangle)) DEALLOCATE (spotstmp%midangle)
IF (ASSOCIATED(spotstmp%sumffact)) DEALLOCATE (spotstmp%sumffact)
IF (ASSOCIATED(spotstmp%arcffact)) DEALLOCATE (spotstmp%arcffact)
IF (ASSOCIATED(spotstmp%tgpoint)) DEALLOCATE (spotstmp%tgpoint)
spotstmp%nspots=0

IF (ASSOCIATED(planetstmp%id)) DEALLOCATE(planetstmp%id)
IF (ASSOCIATED(planetstmp%y)) DEALLOCATE(planetstmp%y)
IF (ASSOCIATED(planetstmp%phi)) DEALLOCATE(planetstmp%phi)
IF (ASSOCIATED(planetstmp%filter)) DEALLOCATE(planetstmp%filter)
IF (ASSOCIATED(planetstmp%dim)) DEALLOCATE(planetstmp%dim)
IF (ASSOCIATED(planetstmp%roots)) DEALLOCATE (planetstmp%roots)
IF (ASSOCIATED(planetstmp%fgarc)) DEALLOCATE (planetstmp%fgarc)
IF (ASSOCIATED(planetstmp%nroots)) DEALLOCATE (planetstmp%nroots)
IF (ASSOCIATED(planetstmp%midangle)) DEALLOCATE (planetstmp%midangle)
IF (ASSOCIATED(planetstmp%sumffact)) DEALLOCATE (planetstmp%sumffact)
IF (ASSOCIATED(planetstmp%arcffact)) DEALLOCATE (planetstmp%arcffact)
IF (ASSOCIATED(planetstmp%tgpoint)) DEALLOCATE (planetstmp%tgpoint)
planetstmp%nplanets=0

END SUBROUTINE DEALLOCATE_TEMPORARY_VECTORS

SUBROUTINE PLOT(thisiter,simp,starp,startmp,planetstmp,spotstmp)
USE getInputs

INTEGER           :: i, iter, thisiter
TYPE(startmp_)    :: startmp
TYPE(spotstmp_)   :: spotstmp
TYPE(planetstmp_) :: planetstmp
TYPE(starparam_)  :: starp
TYPE(simparam_)   :: simp
CHARACTER(len=32) :: arg

IF (iargc()>0) THEN
 CALL getarg(1, arg)
 arg=TRIM(arg)
 IF (arg=="-v") THEN
   IF (iargc()>1) THEN
    CALL getarg(2, arg)
    arg=TRIM(arg)
    READ(arg,'(I10)') iter

    IF (iter>=1.AND.iter<=simp%nsteps) THEN
     IF (iter==thisiter) THEN
      CALL SNAPSHOT(startmp,spotstmp,planetstmp,starp)            
     ENDIF
    ELSE
     CALL ERRMSG(37)
     STOP
    ENDIF 
   ELSE
     IF (thisiter==1) THEN
      CALL SNAPSHOT(startmp,spotstmp,planetstmp,starp)            
     ENDIF
   ENDIF
 ELSE
    CALL ERRMSG(37)
    STOP
 ENDIF
ENDIF

END SUBROUTINE PLOT

SUBROUTINE SNAPSHOT(startmp,spotstmp,planetstmp,starparam)            
IMPLICIT NONE
      
INTEGER, PARAMETER :: niter=1000
INTEGER            :: i,j,flag,iter,nobj
TYPE(startmp_)     :: startmp
TYPE(spotstmp_)    :: spotstmp
TYPE(planetstmp_)  :: planetstmp
REAL(DP)           :: theta,x,y,x_s,y_s,phi

INTEGER, PARAMETER :: nitergrid=360,npar=18,nmer=36 
REAL(DP)           :: z,lat,long,posang,incl
TYPE(starparam_)   :: starparam

 print*,"############################################################"
 print*,"#                  ARC TABLE                               #"
 print*,"############################################################"

IF (startmp%nroots>0) THEN
   print*," OBJ=STELLAR BORDER                                       "
   print*,"                                                          "
   print*,"      ANGLE_IN      ANGLE_FIN      FFARC       FLAG       "
   print*,"                                                          "      
   print*,"                                                          "
   IF (startmp%nroots==1) THEN
       write(*,1000) deg(startmp%roots(1)), deg(TWPI), startmp%arcffact(1), startmp%fgarc(1)
   ELSE
    DO i=1, startmp%nroots-1
       write(*,1000) deg(startmp%roots(i)), deg(startmp%roots(i+1)), startmp%arcffact(i), startmp%fgarc(i)
    ENDDO   
       write(*,1000) deg(startmp%roots(startmp%nroots)), deg(startmp%roots(1)), & 
                     startmp%arcffact(startmp%nroots), startmp%fgarc(startmp%nroots)
   ENDIF
   print*,"                                                          "
ENDIF 

IF (spotstmp%nspots>0) THEN
 print*,"############################################################"
 DO i=1, spotstmp%nspots
  IF (spotstmp%nroots(i)>0) THEN
   print*," OBJ=",spotstmp%id(i)," SPOT                              "
   print*,"                                                          "
   print*,"      ANGLE_IN      ANGLE_FIN       FFARC      FLAG       "
   print*,"                                                          "      
   IF (spotstmp%nroots(i)==1) THEN
      write(*,1000) deg(spotstmp%roots(i,1)), deg(TWPI), spotstmp%arcffact(i,1), spotstmp%fgarc(i,1)
   ELSE
    DO j=1, spotstmp%nroots(i)-1
      write(*,1000) deg(spotstmp%roots(i,j)), deg(spotstmp%roots(i,j+1)), spotstmp%arcffact(i,j), spotstmp%fgarc(i,j)
    ENDDO   
    write(*,1000) deg(spotstmp%roots(i,spotstmp%nroots(i))), deg(spotstmp%roots(i,1)), & 
                  spotstmp%arcffact(i,spotstmp%nroots(i)), spotstmp%fgarc(i,spotstmp%nroots(i))
   ENDIF
   print*,"                                                          "
  ENDIF
 ENDDO
ENDIF

IF (planetstmp%nplanets>0) THEN
 print*,"############################################################"
 DO i=1, planetstmp%nplanets
  IF (planetstmp%nroots(i)>0) THEN
   print*," OBJ=",planetstmp%id(i)," PLANET                          "
   print*,"                                                          "
   print*,"       ANGLE_IN      ANGLE_FIN        FFARC      FLAG     "
   print*,"                                                          "       
   IF (planetstmp%nroots(i)==1) THEN
       write(*,1000) deg(planetstmp%roots(i,1)), deg(TWPI), planetstmp%arcffact(i,1), planetstmp%fgarc(i,1)
   ELSE
    DO j=1, planetstmp%nroots(i)-1
       write(*,1000) deg(setangle(planetstmp%roots(i,j)+planetstmp%phi(i))), & 
                     deg(setangle(planetstmp%roots(i,j+1)+planetstmp%phi(i))), &
                     planetstmp%arcffact(i,j), planetstmp%fgarc(i,j)
    ENDDO   
    write(*,1000) deg(setangle(planetstmp%roots(i,planetstmp%nroots(i))+planetstmp%phi(i))), & 
                  deg(setangle(planetstmp%roots(i,1)+planetstmp%phi(i))), & 
                  planetstmp%arcffact(i,planetstmp%nroots(i)), planetstmp%fgarc(i,planetstmp%nroots(i))
   ENDIF
   print*,"                                                          "
  ENDIF
 ENDDO
ENDIF


!
!    Arcs.
!      

         OPEN(10,file='arcs.dat',status='replace')

         WRITE(10,'(A7,I7,I7,I7)') 'Arcs  =',4,niter+planetstmp%nplanets*niter+spotstmp%nspots*niter+3,1+ & 
                                               planetstmp%nplanets+spotstmp%nspots
         WRITE(10,'(A7,I7,I7)') 'Grid  =',3+niter+planetstmp%nplanets*niter+spotstmp%nspots*niter+1, &
                                          3+niter+planetstmp%nplanets*niter+spotstmp%nspots*niter+1+ & 
                                          npar*nitergrid+nmer*nitergrid-1
         WRITE(10,'(A7,I7)')    'Roots =',3+niter+planetstmp%nplanets*niter+spotstmp%nspots*niter+1+ & 
                                          npar*nitergrid+nmer*nitergrid

         DO iter=1, niter
         theta=real(iter)*(360./real(niter))*PI/180.

         flag=0
         IF (startmp%nroots>0) THEN 
         DO j=1, startmp%nroots

           IF (startmp%nroots==1) then
              flag=startmp%fgarc(j)
           ELSE   
            IF (j==startmp%nroots) THEN
             IF (((theta>startmp%roots(j)).AND.(theta<=TWPI)).OR. &
                 ((theta>=0.).and.(theta<=startmp%roots(1)))) THEN
                  flag=startmp%fgarc(j)
             ENDIF
            ELSE
             IF ((theta>startmp%roots(j)).AND.(theta<=startmp%roots(j+1))) THEN
                 flag=startmp%fgarc(j)
             ENDIF
            ENDIF
           ENDIF

         ENDDO
         ENDIF  
         x=cos(theta)
         y=sin(theta)
         write(10,4000) x,y,flag,1

         ENDDO
         nobj=1



         IF (planetstmp%nplanets>0) THEN
         DO i=1, planetstmp%nplanets
         nobj=nobj+1

         DO iter=1, niter
         theta=real(iter)*(360./real(niter))*PI/180.

         flag=0
         IF (planetstmp%nroots(i)>0) THEN 
          DO j=1, planetstmp%nroots(i)

           IF (planetstmp%nroots(i)==1) then
                   flag=planetstmp%fgarc(i,j)
           ELSE   
            IF (j==planetstmp%nroots(i)) THEN
             IF (((theta>planetstmp%roots(i,j)).AND.(theta<=TWPI)).OR. &
                 ((theta>=0.).and.(theta<=planetstmp%roots(i,1)))) THEN
                   flag=planetstmp%fgarc(i,j)
             ENDIF
            ELSE
             IF ((theta>planetstmp%roots(i,j)).AND.(theta<=planetstmp%roots(i,j+1))) THEN
                   flag=planetstmp%fgarc(i,j)
             ENDIF
            ENDIF
           ENDIF

          ENDDO
         ENDIF  

         x=planetstmp%dim(i)*cos(theta)
         y=planetstmp%dim(i)*sin(theta)+planetstmp%y(i)
         phi=planetstmp%phi(i)

         CALL BACKWARD(x,y,x_s,y_s,phi)
         WRITE(10,4000) x_s,y_s,flag,nobj
         ENDDO  
 
         ENDDO
         ENDIF

  


         IF (spotstmp%nspots>0) THEN
         DO i=1, spotstmp%nspots
         nobj=nobj+1

         DO iter=1, niter
         theta=real(iter)*(360./real(niter))*PI/180.

          flag=0
          IF (spotstmp%nroots(i)>0) THEN 
          DO j=1, spotstmp%nroots(i)

           IF (spotstmp%nroots(i)==1) then
                   flag=spotstmp%fgarc(i,j)
           ELSE   
            IF (j==spotstmp%nroots(i)) THEN
             IF (((theta>spotstmp%roots(i,j)).AND.(theta<=TWPI)).OR. &
                 ((theta>=0.).and.(theta<=spotstmp%roots(i,1)))) THEN
                   flag=spotstmp%fgarc(i,j)
             ENDIF
            ELSE
             IF ((theta>spotstmp%roots(i,j)).AND.(theta<=spotstmp%roots(i,j+1))) THEN
                   flag=spotstmp%fgarc(i,j)
             ENDIF
            ENDIF
           ENDIF

          ENDDO
          ENDIF  

         x=sin(spotstmp%dim(i))*sin(theta)
         y=sin(spotstmp%y(i))*cos(spotstmp%dim(i))-cos(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(theta)
         phi=spotstmp%phi(i)

         call BACKWARD(x,y,x_s,y_s,phi)
         write(10,4000) x_s,y_s,flag,nobj
         ENDDO

 
         ENDDO
         ENDIF


! Stellar grid

posang=setangle(rad(starparam%posang)-HPI)
  incl=rad(starparam%incl)

DO i=1, npar
   lat=rad(90.0D0-dble(i)*10.0D0)
   DO iter=1, nitergrid
      long=rad(dble(iter))
      x=cos(posang)*(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))+cos(lat)*sin(long)*sin(posang) 
      y=cos(lat)*sin(long)*cos(posang)-(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))*sin(posang)
      z=sin(lat)*cos(incl)-cos(lat)*cos(long)*sin(incl)
      IF (z>=0.0D0) THEN
          WRITE(10,2000) x,y,iter,1 
      ELSE
          WRITE(10,2000) x,y,iter,0
      ENDIF
   ENDDO
ENDDO

DO i=1, nmer
   long=rad(dble(i)*(360./dble(nmer)))
   DO iter=1, nitergrid
      lat=rad(90.0D0-dble(iter)/2.0D0)
      x=cos(posang)*(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))+cos(lat)*sin(long)*sin(posang) 
      y=cos(lat)*sin(long)*cos(posang)-(cos(lat)*cos(long)*cos(incl)+sin(lat)*sin(incl))*sin(posang)
      z=sin(lat)*cos(incl)-cos(lat)*cos(long)*sin(incl)
      IF (z>=0.0D0) THEN
          WRITE(10,2000) x,y,iter,2
      ELSE
          WRITE(10,2000) x,y,iter,0
      ENDIF
   ENDDO
ENDDO


! Roots

      IF (startmp%nroots>0) THEN
         DO i=1, startmp%nroots
            IF (startmp%fgarc(i)==1) THEN

                x=cos(startmp%roots(i))
                y=sin(startmp%roots(i))
                WRITE(10,3000) x, y

            ENDIF
         ENDDO
      ENDIF

      IF (planetstmp%nplanets>0) THEN      
       DO i=1, planetstmp%nplanets
       IF (planetstmp%nroots(i)>0) THEN
         DO j=1, planetstmp%nroots(i)
            IF (planetstmp%fgarc(i,j)==1) THEN

                x=planetstmp%dim(i)*cos(planetstmp%roots(i,j))
                y=planetstmp%dim(i)*sin(planetstmp%roots(i,j))+planetstmp%y(i)
                phi=planetstmp%phi(i)
                call BACKWARD(x,y,x_s,y_s,phi)
                WRITE(10,3000) x_s, y_s

            ENDIF
         ENDDO
       ENDIF
       ENDDO
      ENDIF

      IF (spotstmp%nspots>0) THEN      
       DO i=1, spotstmp%nspots
        IF (spotstmp%nroots(i)>0) THEN
         DO j=1, spotstmp%nroots(i)
            IF (spotstmp%fgarc(i,j)==1) THEN

                x=sin(spotstmp%dim(i))*sin(spotstmp%roots(i,j))
                y=sin(spotstmp%y(i))*cos(spotstmp%dim(i))-cos(spotstmp%y(i))*sin(spotstmp%dim(i))*cos(spotstmp%roots(i,j))
                phi=spotstmp%phi(i)
                call BACKWARD(x,y,x_s,y_s,phi)
                WRITE(10,3000) x_s, y_s

            ENDIF
         ENDDO
        ENDIF
       ENDDO
      ENDIF

CLOSE(10)

1000 FORMAT(3(5X,F9.3),7X,I7)
2000 format(2(2X,F9.3),2(2X,I7))
3000 format(2(2X,F15.6),2(2X,I7))
4000 format(2(2X,F15.6),3(2X,I7))
END SUBROUTINE SNAPSHOT

END MODULE Maths ! =====================================================
